package edu.emory.mathcs;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import edu.emory.mathcs.measures.privacy.DisclosureProbability;

public class CorruptionAttack
{
	public static Map<Set<Site>, Map<Tuple, Double>> getProbabilities(TuplesGroup tuples, int numAttackers)
	{
		Map<Set<Site>, Map<Tuple, Double>> values = new HashMap<Set<Site>, Map<Tuple, Double>>();
		final Map<Site, List<Tuple>> site2tuples = tuples.getTuplesSplittedToSites();
		CombinationsGenerator<Site> cg = new CombinationsGenerator<Site>(site2tuples.keySet());

		cg.setCardinality(numAttackers);
		
		while (cg.hasNext()) {
			Set<Site> attackers = cg.next();
			corruptSitesTuples(tuples, attackers);

			if (tuples.getNumUncorruptedTuples() > 0)
				values.put(attackers, DisclosureProbability.getDisclosureProbabilities(tuples));

			uncorruptSitesTuples(tuples, attackers);
		}
		
		return values;
	}

	public static int uncorruptSitesTuples(TuplesGroup tuples, Set<Site> attackers)
	{
		int numUncorruptedTuples = 0;
		
		for (Tuple t : tuples.getTuples())
		{
			if (attackers.contains(t.getSite())) {
				t.setCorrupted(false);
				numUncorruptedTuples++;
			}
		}
		
		tuples.clearAttributeRanges();
		return numUncorruptedTuples;
	}

	public static int corruptSitesTuples(TuplesGroup tuples, Set<Site> attackers)
	{
		int numCorruptedTuples = 0;
		for (Tuple t : tuples.getTuples())
		{
			if (attackers.contains(t.getSite())) {
				t.setCorrupted(true);
				numCorruptedTuples++;
			}
		}
		
		tuples.clearAttributeRanges();
		return numCorruptedTuples;
	}
	
}
