package edu.emory.mathcs;

public class IntAttributeRange implements Comparable<IntAttributeRange> {
	
	private int min;
	private int max;
	private boolean isEmpty;
	
	public IntAttributeRange(int min, int max) {
		this.min = min;
		this.max = max;
		this.isEmpty = (min > max);
	}

	public IntAttributeRange(IntAttributeRange src) {
		this.min = src.getMin();
		this.max = src.getMax();
		this.isEmpty = src.isEmpty();
	}

	public IntAttributeRange(String s) {
		String[] nums = s.split("-");
		
		this.min = Integer.parseInt(nums[0]);
		this.max = Integer.parseInt(nums[1]);
		this.isEmpty = (min > max);
	}
	
	public boolean isEmpty()
	{
		return this.isEmpty;
	}

	public int getMin() {
		return min;
	}
	
	public int getMax() {
		return max;
	}
	
	public double getMedian(Object[] data) {
		throw new RuntimeException("Not implemented");
	}
	
	public int getRange()
	{
		return this.getMax() - this.getMin();
	}
	
	public void setMinMax(int min, int max)
	{
		this.min = min;
		this.max = max;
		this.isEmpty = (min > max);
	}

	@Override
	public int compareTo(IntAttributeRange o) {
		if (this.isEmpty())
			return -1;
		
		if (o.isEmpty())
			return 1;
		
		return this.getMin() - o.getMin();
	}
	
	public boolean isEqualTo(IntAttributeRange o) {
		if (this.isEmpty() && o.isEmpty())
			return true;
		
		if (this.getMin() == o.getMin() && this.getMax() == o.getMax())
			return true;
		
		return false;
	}
	
	public String toString() {
		if (isEmpty())
			return "[-,-]";
		return "[" + this.getMin() + "," + this.getMax() + "]"; 
	}

	public IntAttributeRange intersect(IntAttributeRange range)
	{
		IntAttributeRange intersection = new IntAttributeRange(Math.max(this.getMin(), range.getMin()), Math.min(this.getMax(), range.getMax()));
			
		return intersection;
	}

	public void reverseEnds()
	{
		this.setMinMax(this.getMax(), this.getMin());
	}

	public boolean isWithin(int data)
	{
		return (this.getMin() <= data && this.getMax() >= data);
	}

	public void updateFor(int value)
	{
		this.min = Math.min(this.min, value);
		this.max = Math.max(this.max, value);
		this.isEmpty = (this.min > this.max);
	}
}
