package edu.emory.mathcs;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import edu.emory.mathcs.measures.utility.QueryError.generation;

public class Query {

	private List<QueryRange> ranges;
	private static final Random rand = new Random();
	private static final double DEFAULT_RANGE = 0.25;

	public Query(int predicates, generation random, RawTuples stats) {
		this.ranges = null;
		
		if (random == generation.RANDOM) {
			this.ranges = Query.generateRandomQuery(predicates, stats);
		}
	}

	private static List<QueryRange> generateRandomQuery(int numPredicates,
			RawTuples stats) {
		
		final int numQIDs = stats.getNumQIDAttributes();
		if (numPredicates > numQIDs) {
			throw new RuntimeException("Error");
		}
		List<QueryRange> ranges = new ArrayList<QueryRange>(numPredicates);
		int[] queriedAttributes = new int[numPredicates];
		List<Integer> allAttributes = new ArrayList<Integer>(numQIDs);
		for (int qid = 0; qid < numQIDs; qid++)
			allAttributes.add(qid);

		for (int generated = 0; generated < numPredicates; generated++)
			queriedAttributes[generated] = allAttributes.remove(rand.nextInt(allAttributes.size()));
		
//		main: while (true) {
//			queriedAttributes[generated] = rand.nextInt(numQIDs);
//			for (int i = 0; i < generated; i++) {
//				if (queriedAttributes[i] == queriedAttributes[generated]) {
//					continue main;
//				}
//			}
//			generated++;
//			if (generated == queriedAttributes.length) {
//				break;
//			}
//		}
		for (int i = 0; i < queriedAttributes.length; i++) {
			IntAttributeRange attributeRange = stats.getAttributeDomain(queriedAttributes[i]);
			int spread = attributeRange.getRange();
			if (spread == 1) {
				// this is boolean value
				int val;
				if (rand.nextBoolean()) {
					val = attributeRange.getMax();
				} else {
					val = attributeRange.getMin();
				}
				ranges.add(new QueryRange(queriedAttributes[i], val, val));
			} else {
				int begin = rand.nextInt((int) (spread * (1 - DEFAULT_RANGE))) + attributeRange.getMin();
				ranges.add(new QueryRange(queriedAttributes[i], begin, (int) Math.min(Math.round(begin + DEFAULT_RANGE * spread), attributeRange.getMax())));
			}
		}
		return ranges;
	}

	public QueryRange[] getRanges() {
		return this.ranges.toArray(new QueryRange[0]);
	}

	public QueryRange getRange(int i) {
		return this.ranges.get(i);
	}

	public int getNumRanges() {
		return this.ranges.size();
	}

}
