package edu.emory.mathcs;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import edu.emory.mathcs.measures.Measure;
import edu.emory.mathcs.privacy.PrivacyChecker;

public class SingleSiteCorruptionAttack {

	public static Map<Site, Boolean> arePrivate(TuplesGroup tuples, PrivacyChecker checker)
	{
		Map<Site, Boolean> arePrivate = new HashMap<Site, Boolean>();
		Set<Site> sites = tuples.getSiteFrequencies().keySet();
		
		for (Site site : sites)
		{
			corruptSiteTuples(tuples, site);
			
			arePrivate.put(site, checker.isAnonymized(tuples));
			
			uncorruptSiteTuples(tuples, site);
		}
		
		return arePrivate;
	}
	
	public static Map<Site, Double> getValue(TuplesGroup tuples, Measure m)
	{
		Map<Site, Double> values = new HashMap<Site, Double>();
		Set<Site> sites = tuples.getSiteFrequencies().keySet();
		
		for (Site site : sites)
		{
			corruptSiteTuples(tuples, site);
			
			values.put(site, m.getValue(tuples));
			
			uncorruptSiteTuples(tuples, site);
		}
		
		return values;
	}
	
	private static TuplesGroup corruptSiteTuples (TuplesGroup tuples, Site site)
	{
		for (Tuple t : tuples.getTuplesFromSite(site))
			t.setCorrupted(true);
		
		tuples.clearAttributeRanges();
		
		return tuples;
	}

	private static TuplesGroup uncorruptSiteTuples (TuplesGroup tuples, Site site)
	{
		for (Tuple t : tuples.getTuplesFromSite(site))
			t.setCorrupted(false);

		tuples.clearAttributeRanges();

		return tuples;
	}
	
	public static int getNumberDisclosingSites(TuplesGroup tuples, PrivacyChecker checker)
	{
		Map<Site, Boolean> arePrivate = arePrivate(tuples, checker);
		
		int counter = 0;
		for (Boolean b : arePrivate.values())
			if (b == false)
				counter++;
		
		return counter;
	}
}
