package edu.emory.mathcs;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

public class Tools {

	public static double sumUpAttributeValues(TuplesGroup tuples, int attributeIndex) {
		double sum = 0.0;

		if (tuples.hasSubGroups()) {
			sum = Tools.sumUpAttributeValues(tuples.getLeftChild(), attributeIndex)
					+ Tools.sumUpAttributeValues(tuples.getRightChild(), attributeIndex);
		} else {
			for (Tuple tuple : tuples.getData()) 
			{
				sum += tuple.getData(attributeIndex);
			}
		}

		return sum;
	}

	public static double avgAttributeValues(TuplesGroup tuples,
			int attributeIndex) {
		return Tools.sumUpAttributeValues(tuples, attributeIndex)
				/ tuples.getNumTuples();
	}

	public static double getProjectedTo01(double min, double max, double val) {
		return (val - min) / (max - min);
	}
	
	public static double findMedian(int bestAttr, List<Tuple> data) {

		List<Data> items = new ArrayList<Data>();
		for (int i = 0; i < data.size(); i++) {
			items.add(new Data(data.get(i).getData()[bestAttr]));
		}
		Collections.sort(items, new Comparator<Data>() {
			public int compare(Data o1, Data o2) {
				if (o1.value < o2.value) {
					return 1;
				} else if (o1.value > o2.value) {
					return -1;
				} else {
					return 0;
				}
			}
		});

		double median;
		if (data.size() % 2 == 0) {
			int i = data.size() / 2;
			median = (items.get(i).value + items.get(i - 1).value) / 2.0;
		} else {
			median = items.get(data.size() / 2).value;
		}
		return median;
	}
	
	public static double findMedian(int bestAttr, Tuple[] data) {

		List<Data> items = new ArrayList<Data>();
		for (int i = 0; i < data.length; i++) {
			items.add(new Data(data[i].getData()[bestAttr]));
		}
		Collections.sort(items, new Comparator<Data>() {
			public int compare(Data o1, Data o2) {
				if (o1.value < o2.value) {
					return 1;
				} else if (o1.value > o2.value) {
					return -1;
				} else {
					return 0;
				}
			}
		});

		double median;
		if (data.length % 2 == 0) {
			int i = data.length / 2;
			median = (items.get(i).value + items.get(i - 1).value) / 2.0;
		} else {
			median = items.get(data.length / 2).value;
		}
		return median;
	}

	private static class Data {

		int value;

		public Data(int value) {
			this.value = value;
		}

	}

	public static Map<Integer, Integer> getFrequencies(TuplesGroup tuples, int attributeIndex) {
		
		Map<Integer, Integer> freqs = new TreeMap<Integer, Integer>();
		
		for (Tuple tuple : tuples.getTuples())
		{
			int value = tuple.getData(attributeIndex);
			Integer currentFreq = freqs.get(value);
			if (currentFreq == null)
				currentFreq = 0;
			
			freqs.put(value, currentFreq + 1);
		}
		
		return freqs;
	}

//	public static Map<Integer, Integer> getFrequencies(TuplesGroup1 tuples, int attributeIndex) {
//		
//		Map<Integer, Integer> freqs = new TreeMap<Integer, Integer>();
//		
//		for (Tuple tuple : tuples.getTuples())
//		{
//			int value = tuple.getData(attributeIndex);
//			Integer currentFreq = freqs.get(value);
//			if (currentFreq == null)
//				currentFreq = 0;
//			
//			freqs.put(value, currentFreq + 1);
//		}
//		
//		return freqs;
//	}

	public static Map<Integer, Integer> getUncorruptedFrequencies(TuplesGroup tuples, int attributeIndex) {
		
		Map<Integer, Integer> freqs = new TreeMap<Integer, Integer>();
		
		Tuple[] ts = tuples.getTuples();
		if (ts == null || ts.length == 0)
			return freqs;
		
		for (Tuple tuple : ts)
		{
			if (tuple.isCorrupted())
				continue;
			
			int value = tuple.getData(attributeIndex);
			Integer currentFreq = freqs.get(value);
			if (currentFreq == null)
				currentFreq = 0;
			
			freqs.put(value, currentFreq + 1);
		}
		
		return freqs;
	}

//	public static Map<Integer, Integer> getUncorruptedFrequencies(TuplesGroup1 tuples, int attributeIndex) {
//		
//		Map<Integer, Integer> freqs = new TreeMap<Integer, Integer>();
//		
//		Tuple[] ts = tuples.getTuples();
//		if (ts == null || ts.length == 0)
//			return freqs;
//		
//		for (Tuple tuple : ts)
//		{
//			if (tuple.isCorrupted())
//				continue;
//			
//			int value = tuple.getData(attributeIndex);
//			Integer currentFreq = freqs.get(value);
//			if (currentFreq == null)
//				currentFreq = 0;
//			
//			freqs.put(value, currentFreq + 1);
//		}
//		
//		return freqs;
//	}

	public static Map<Integer, Double> getValueProbabilities(TuplesGroup tuples, int attributeIndex, boolean onlyUncorrupted)
	{
		Map<Integer, Integer> freqs = null;
		
		if (onlyUncorrupted)
			freqs = Tools.getUncorruptedFrequencies(tuples, attributeIndex);
		else
			freqs = Tools.getFrequencies(tuples, attributeIndex);
		
		double sum = 0.0;
		for (Integer f : freqs.values())
			sum += f;
		
		Map<Integer, Double> probs = new TreeMap<Integer, Double>();
		
		for (Integer value : freqs.keySet())
		{
			Integer freq = freqs.get(value);
			if (freq == 0)
				continue;
			
			probs.put(value, (double) freq / sum);
		}
		
		return probs;
	}

//	public static Map<Integer, Double> getValueProbabilities(TuplesGroup1 tuples, int attributeIndex, boolean onlyUncorrupted)
//	{
//		Map<Integer, Integer> freqs = null;
//		
//		if (onlyUncorrupted)
//			freqs = Tools.getUncorruptedFrequencies(tuples, attributeIndex);
//		else
//			freqs = Tools.getFrequencies(tuples, attributeIndex);
//		
//		double sum = 0.0;
//		for (Integer f : freqs.values())
//			sum += f;
//		
//		Map<Integer, Double> probs = new TreeMap<Integer, Double>();
//		
//		for (Integer value : freqs.keySet())
//		{
//			Integer freq = freqs.get(value);
//			if (freq == 0)
//				continue;
//			
//			probs.put(value, (double) freq / sum);
//		}
//		
//		return probs;
//	}

	/**
	 * Get entropy value from a collection which value frequencies are given using formula:
	 * 		E = - sum_{number} P(number) * ln(P(number)) 
	 * where P(number) is a probability of randomly choosing a number from numbers
	 * P(number) = # number / # numbers
	 * @param freqs frequency of elements which entropy will be computed
	 * @return entropy value
	 */
	public static double getEntropy(Collection<? extends Number> freqs) {
		int size = freqs.size();
		if (size == 0)
			return Double.NaN;
		
		double entropy = 0.0;
		for(Number freq : freqs)
		{
			entropy -= ((freq.doubleValue() / size) * Math.log(freq.doubleValue() / size));  
		}
		
		return entropy;
	}
	
	public static <T> Set<Set<T>> getPowerSet(Set<T> set)
	{
		Set<Set<T>> powerSet = new HashSet<Set<T>>();
		
		for (int i = 0; i <= set.size(); i++)
			powerSet.addAll(getPowerSet(set, i));
		
		return powerSet;
	}

	public static <T> Set<Set<T>> getPowerSet(final Set<T> set, final int numElements) 
	{
		Set<Set<T>> powerSet = new HashSet<Set<T>>();
		Set<T> current = new HashSet<T>();
		
		return Tools.getPowerSet(new ArrayList<T>(set), numElements, powerSet, current);
	}
	
	private static <T> Set<Set<T>> getPowerSet(final List<T> set, final int numElements, Set<Set<T>> power, Set<T> current) 
	{	

		if (current.size() == numElements) {
			
			power.add(new HashSet<T>(current));
			return power;
		}

		for (int i = 0; i < set.size(); i++)
		{
			T a = set.get(i);
			if (current.contains(a))
				continue;
			current.add(a);
			
			power = getPowerSet(set, numElements, power, current);
			
			current.remove(a);
		}
		
		return power;
	}
	
	@Deprecated
	public static <T> Set<Set<T>> getPowerSet_old(Set<T> set, int numElements)
	{
		if (set == null)
			return null;
		
		ArrayList<T> list = new ArrayList<T>(set);
		
		Set<Set<T>> powerSet = new HashSet<Set<T>>();
		// add empty set
		if (numElements == 0)
		{
			powerSet.add(new HashSet<T>());
			return powerSet;
		}
		
		long bits = 0;
		for (int i = set.size() - numElements; i < set.size(); i++)
			bits += (long)Math.pow(2, i);
		
		long currentBits;
		Set<T> currentSet = null;
		int i = 0;
		for (;bits > 0; bits--)
		{
			if (Tools.countBits(bits) != numElements)
				continue;
			
			currentBits = bits;
			i = 0;
			currentSet = new HashSet<T>();
			while (currentBits > 0)
			{
				if ((currentBits & 1) > 0)
					currentSet.add(list.get(i));
				
				currentBits = currentBits >> 1;
				i++;
			}
			
			powerSet.add(currentSet);
		}
		
		return powerSet;
	}

	public static <T> Set<Set<T>> getPowerSetB(Set<T> set, int numElements)
	{
		if (set == null)
			return null;
		
		Set<Set<T>> powerSet = new HashSet<Set<T>>();
		// add empty set
		if (numElements == 0)
		{
			powerSet.add(new HashSet<T>());
			return powerSet;
		}
		
		ArrayList<T> list = new ArrayList<T>(set);
		BitSet bitS = new BitSet(list.size());
		bitS.set(list.size() - numElements, list.size(), true);
		
		Set<T> currentSet = null;
		while (bitS.isEmpty() == false)
		{
			if (bitS.cardinality() != numElements)
			{
				bitS.flip(0, bitS.nextSetBit(0) + 1);
				continue;
			}
			
			currentSet = new HashSet<T>();
			for (int i = bitS.nextSetBit(0); i >= 0; i = bitS.nextSetBit(i+1)) {
				 currentSet.add(list.get(i));
			}
			powerSet.add(currentSet);
			
			if (bitS.cardinality() > 0)
				bitS.flip(0, bitS.nextSetBit(0) + 1);
		}
		
		return powerSet;
	}

	public static int countBits(long x)
	{
		int count = 0;
		while (x != 0)
		{
			if (x % 2 == 1)
				count++;
			x = x >> 1;
		}
		return count;
	}

	public static ArrayList<Tuple> getUncorruptedTuples(ArrayList<Tuple> tuples)
	{
		ArrayList<Tuple> ts = new ArrayList<Tuple>();
		
		for (Tuple t : tuples)
			if (t.isCorrupted() == false)
				ts.add(t);
		
		return ts;
	}

//	public static Map<Site, Integer> getSitesFrequencies(TuplesGroup1 tuples) {
//		return Tools.getSitesFrequencies(tuples.getTuplesAsList());
//	}

	public static Map<Site, Integer> getSitesFrequencies(TuplesGroup tuples) {
		return Tools.getSitesFrequencies(tuples.getTuplesAsList());
	}

	public static Map<Site, Integer> getSitesFrequencies(List<Tuple> tuples) {
		Map<Site, Integer> freqs = new TreeMap<Site, Integer>();
		
		Tuple[] ts = tuples.toArray(new Tuple[0]);
		if (ts == null || ts.length == 0)
			return freqs;
		
		for (Tuple tuple : ts)
		{
			if (tuple.isCorrupted())
				continue;
			
			Site site = tuple.getSite();
			
			if (site == null)
				continue;
			
			Integer currentFreq = freqs.get(site);
			if (currentFreq == null)
				currentFreq = 0;
			
			freqs.put(site, currentFreq + 1);
		}
		
		return freqs;
	}

	public static double getMin(Map<?, Double> values)
	{
		double min = Double.NaN;
		
		if (values == null)
			return Double.NaN;
		
		for (Double d : values.values())
		{
			if (Double.isNaN(d))
				continue;

			if (Double.isNaN(min) || d < min)
				min = d;
		}
			
		return min;
	}
	
	public static double getAvg(Map<?, Double> values)
	{
		double sum = 0.0;
		
		if (values == null)
			return Double.NaN;
		
		for (Double d : values.values())
		{
			if (Double.isNaN(d))
				continue;

			sum += d;
		}
			
		return sum / (double) values.size();
	}
	
	public static double getMax(Map<?, Double> values)
	{
		double max = Double.NaN;
		
		if (values == null)
			return Double.NaN;
		
		for (Double d : values.values())
		{
			if (Double.isNaN(d))
				continue;
			
			if (Double.isNaN(max) || d > max)
				max = d;
		}
			
		return max;
	}


	public static double getMinMin(Map<?, Map<Tuple, Double>> site2tuple2prob)
	{
		Double minmin = Double.NaN;
		
		if (site2tuple2prob == null)
			return Double.NaN;
		
		for (Object o : site2tuple2prob.keySet()) {
			Double min = Tools.getMin(site2tuple2prob.get(o));
			
			if (Double.isNaN(min))
				continue;
			
			if (Double.isNaN(minmin) || min < minmin)
				minmin = min;
		}
		
		return minmin;
	}
	
	public static double getMaxMax(Map<?, Map<Tuple, Double>> site2tuple2probs)
	{
		Double maxmax = Double.NaN;
		
		if (site2tuple2probs == null)
			return Double.NaN;
		
		for (Object o : site2tuple2probs.keySet()) {
			Double max = Tools.getMax(site2tuple2probs.get(o));
			
			if (Double.isNaN(max))
				continue;
			
			if (Double.isNaN(maxmax) || max > maxmax)
				maxmax = max;
		}
		
		return maxmax;
	}
	
	public static double getAvgAvg(Map<?, Map<Tuple, Double>> site2tuple2probs)
	{
		Double avgavg = 0.0;
		
		if (site2tuple2probs == null)
			return Double.NaN;
		
		for (Object o : site2tuple2probs.keySet()) {
			Double avg = Tools.getAvg(site2tuple2probs.get(o));
			
			if (Double.isNaN(avg))
				continue;
			
			avgavg += avg;
		}
		
		return avgavg / site2tuple2probs.size();
	}

	public static double getMinForTuple(Map<Site, Map<Tuple, Double>> site2tuple2prob, Tuple t)
	{
		Map<Site, Double> site2probs = new HashMap<Site,Double>();
		for (Site site : site2tuple2prob.keySet())
		{
			Map<Tuple, Double> tuple2probs = site2tuple2prob.get(site);
			if (tuple2probs.containsKey(t))
				site2probs.put(site, tuple2probs.get(t));
		}
		return Tools.getMin(site2probs);
	}
	
	public static double getMaxForTuple(Map<Site, Map<Tuple, Double>> site2tuple2prob, Tuple t)
	{
		Map<Site, Double> site2probs = new HashMap<Site,Double>();
		for (Site site : site2tuple2prob.keySet())
		{
			Map<Tuple, Double> tuple2probs = site2tuple2prob.get(site);
			if (tuple2probs.containsKey(t))
				site2probs.put(site, tuple2probs.get(t));
		}
		return Tools.getMax(site2probs);
	}
	
	public static double getAvgForTuple(Map<Site, Map<Tuple, Double>> site2tuple2prob, Tuple t)
	{
		Map<Site, Double> site2probs = new HashMap<Site,Double>();
		for (Site site : site2tuple2prob.keySet())
		{
			Map<Tuple, Double> tuple2probs = site2tuple2prob.get(site);
			if (tuple2probs.containsKey(t))
				site2probs.put(site, tuple2probs.get(t));
		}
		return Tools.getAvg(site2probs);
	}
}