package edu.emory.mathcs;

import java.util.Map;
import java.util.Random;
import java.util.TreeMap;

public class Tuple {
	
	public static final int RANDOM_UNIFORM = 1;
	public static final int RANDOM_GAUSIAN = 2;
	public static final int RANDOM_PARTITIONED = 3;
	public static final int RANDOM_ONE_MAJOR = 4;
	public static final int RANDOM_ATTR_TO_SITE = 5;
	public static final int RANDOM_BA_MODEL = 6;
	
	//public static final int N = Main.CURR_N;
	public double R = 4;
	
	
	private static int majorSite = -1;
	
	private static int attributeNumber = -1;
	private static Map<Integer, Integer> attributeValue2SiteID = null;
	private static int nextSite = 0;
	
	private static Random rand = new Random();
	
	private int[] data;
	private Site site;
	private boolean isCorrupted;
	
	public Tuple(int[] data, Site site) {
		this.data = data;
		this.isCorrupted = false;
		this.site = site;
	}
	
	public void setSite(int randType, int numOwners)
	{
		if (this.site != null)
			this.site.removeTuple(this);
		
		if (randType == RANDOM_UNIFORM) {
			this.site = SitesFactory.getSite(randomOwner(numOwners));
		} else if (randType == RANDOM_GAUSIAN) {
			this.site = SitesFactory.getSite(randomGausianOwner(numOwners));
		} else if (randType == RANDOM_PARTITIONED) {
			this.site = SitesFactory.getSite(randomPartitionedOwner(data,numOwners));
		} else if (randType == RANDOM_ONE_MAJOR) {
			if (Tuple.majorSite == -1)
				Tuple.majorSite = (int) (Math.random() * numOwners);
			this.site = SitesFactory.getSite(randomOneMajorOwner(numOwners));
		} else if (randType == RANDOM_ATTR_TO_SITE) {
			if (Tuple.attributeNumber == -1)
				setAttributeValues2SiteMap(data, numOwners);
			this.site = SitesFactory.getSite(randomOneAttrToSiteOwner(numOwners));
		} else {
			throw new RuntimeException("Type undefined: " + randType);
		}
		this.site.addTuple(this);
	}
	
	public void setSite(Site site) 
	{
		this.site = site;
		this.site.addTuple(this);
	}
	
	private void setAttributeValues2SiteMap(int[]data, int numOwners)
	{
		// country!
		Tuple.attributeNumber = 7;//(int) (Math.random() * (data.length - 1));

		Tuple.attributeValue2SiteID = new TreeMap<Integer, Integer>();
		Tuple.nextSite = 0;
	}

	private int randomOneAttrToSiteOwner(int numOwners)
	{
		int highProb = -1;
		final double majority = 0.9;
		
		if (Tuple.attributeValue2SiteID.containsKey(data[Tuple.attributeNumber]))
			highProb = Tuple.attributeValue2SiteID.get(data[Tuple.attributeNumber]);
		else
		{
			highProb = Tuple.nextSite++;
			Tuple.attributeValue2SiteID.put(data[Tuple.attributeNumber], highProb);
		}

		int randNum = (int) (Math.random() * (1.0 + majority) * numOwners);
	
		if (randNum >= numOwners)
			randNum = highProb;
		
		return randNum;
	}
	
	private int randomOneMajorOwner(int numOwners)
	{
		final double majority = 0.8;
		
		int randNum = (int) (Math.random() * (1.0 + majority) * numOwners);
		
		if (randNum >= numOwners)
			randNum = Tuple.majorSite;
		
		return randNum;
	}

	private int randomPartitionedOwner(int[] data2, int numOwners) {
//		int node;
//		if (data2[0] < 25) {
//			node = rand.nextInt(Main.CURR_N / 4);
//		} else if (data2[0] < 35) {
//			node = rand.nextInt(10) + Main.CURR_N / 4;
//		} else if (data2[0] < 45) {
//			node = rand.nextInt(10) + Main.CURR_N / 4 + 10;
//		} else {
//			node = rand.nextInt(Main.CURR_N - 20 - Main.CURR_N / 4) + Main.CURR_N / 4 + 20;
//		}
//		return node;
		
//		attr[1]:
//			 0,1074
//			 1,2499
//			 2,943
//			 3,2067
//			 4,1278
//			 5,22286
//			 6,14

		int step = numOwners / 6;
		int nodeMin = data2[1] * step;
		int nodeMax = (data[1] + 1)* step;
		if (nodeMin >= numOwners) {
			nodeMin = numOwners - 1;
		}
		if (nodeMax >= numOwners) {
			nodeMax = numOwners - 1;
		}
		
		int node;
		//System.out.println("Min: " + nodeMin + "  Max: " + nodeMax);
		int range = (nodeMax - nodeMin) / 3;
		if (range > 0) {
			int lastRange = nodeMax - nodeMin - range * 2 + 1;
			if (data2[0] < 25) {
				node = rand.nextInt(range);
			} else if (data2[0] < 55) {
				node = rand.nextInt(range) + range;
			} else {
				node = rand.nextInt(lastRange) + 2*range;
			}
		} else {
			node = rand.nextInt(nodeMax - nodeMin + 1);
		}
		return node + nodeMin;
		
	}

	private int randomGausianOwner(int numOwners) {
		double r = rand.nextGaussian();
		if (r < -R) {
			r = -R;
		}
		if (r > R) {
			r = R;
		}
		r = (r + R) / R / 2.0;
		//System.out.println("R=" + r);
		int index = (int) (numOwners * r);
		if (index == numOwners) {
			index = index - 1;
		}
		return (int) index;
	}

	private static int randomOwner(int numOwners) {
		return rand.nextInt(numOwners);
	}
	
	public int getSiteID() {
		return this.getSite().getID();
	}
	
	public Site getSite() {
		return this.site;
	}
	
	public String toString() {
		StringBuffer s = new StringBuffer();
		for (int i = 0; i < data.length; i++) {
			s.append("[").append(data[i]).append("]");
		}
		return s.toString();
	}

	public int[] getData() {
		return data;
	}

	public int getData(int index) {
		
		if (index == TuplesGroup.SENSITIVE)
			return this.data[this.data.length - 1];
//			return this.getSiteID(); 
		
		return data[index];
	}

	public int getSensitiveData() {
		return this.getData(TuplesGroup.SENSITIVE);
	}

	public boolean isCorrupted(){
		return this.isCorrupted;
	}

	public void setCorrupted(boolean v){
		this.isCorrupted = v;
	}

	public int getAttributesNumber() {
		return this.data.length - 1;
	}
	
}
