package edu.emory.mathcs;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;


// Experimental (improved performance - tuples are hold in site2tuples map).
public class TuplesGroup1 /*implements BinaryTreeNode<TuplesGroup1> */{
/*	
	public static final int SENSITIVE = -1;
	private static final String LEFT_CHILD = "L";
	private static final String RIGHT_CHILD = "R";
	private static final String DATA = "D";
	private static int currentID = 0;
	private List<Tuple> data;
//	private Set<Site> sites;
	private Map<Site, List<Tuple>> site2Tuples;
	private IntAttributeRange[] attributeRanges;
	
	private TuplesGroup1 parent = null;
	private TuplesGroup1 left = null;
	private TuplesGroup1 right = null;
	private int attributeNumber;
	private int groupID;
	
	public TuplesGroup1(Tuple[] data) {
		this(Arrays.asList(data));
	}

	public TuplesGroup1(Collection<Tuple> data) {
		this(data, null);
		this.attributeRanges = computeAttributeRanges();
	}

	public TuplesGroup1(RawTuples stats) {
		this(stats.getTuples());
	}
	
	public TuplesGroup1(Collection<Tuple> data, Set<Site> sites, IntAttributeRange[] ranges) {
		this.groupID = TuplesGroup1.currentID++;
//		this.sites = new HashSet<Site>();
//		if (sites != null)
//			this.sites.addAll(sites);
		this.site2Tuples = new HashMap<Site, List<Tuple>>();
		this.data = new ArrayList<Tuple>();
		if (data != null)
			this.data.addAll(data);
		this.attributeNumber = (data == null || data.size() == 0 ? 0 : this.data.get(0).getData().length - 1);
		this.attributeRanges = ranges;
		this.left = null;
		this.right = null;
		this.parent = null;
	}
	
	public TuplesGroup1(Collection<Tuple> data, IntAttributeRange[] ranges) {
		this.groupID = TuplesGroup1.currentID++;
		this.sites = new HashSet<Site>();
		if (data != null)
		{
			this.data = new ArrayList<Tuple>(data.size());
			this.data.addAll(data);
			for (Tuple t : data)
				this.sites.add(t.getSite());
		}
		else
		{
			this.data = new ArrayList<Tuple>();
		}
		this.attributeNumber = (data == null || data.size() == 0 ? 0 : this.data.get(0).getData().length - 1);
		this.attributeRanges = ranges;
		this.left = null;
		this.right = null;
		this.parent = null;
	}

	public TuplesGroup1(TuplesGroup1 tuples) {
		this.groupID = TuplesGroup1.currentID++;
		this.sites = new HashSet<Site>();
		if (tuples.data != null)
		{
			this.data = new ArrayList<Tuple>(tuples.data.size());
			this.data.addAll(tuples.data);
			for (Tuple t : this.data)
				this.sites.add(t.getSite());
		}
		else
		{
			this.data = new ArrayList<Tuple>();
		}
		
		this.attributeNumber = tuples.getNumAttributes();
		this.attributeRanges = Arrays.copyOf(tuples.getAttributeRanges(), tuples.getAttributeRanges().length);
		this.parent = tuples.getParent();
		
		if (tuples.getLeftChild() != null) {
			this.left = new TuplesGroup1(tuples.getLeftChild());
		}
		if (tuples.getRightChild() != null) {
			this.right = new TuplesGroup1(tuples.getRightChild());
		}
		
	}

	public Set<Integer> getDomain(int attributeID, boolean onlyUncorrupted) {
		Map<Integer, Double> values2prob = this.getAttributeValueProbabilities(attributeID, onlyUncorrupted);
		return values2prob.keySet();
	}
	
	public boolean doOverlap(TuplesGroup1 group)
	{
		boolean doOverlap = true;
		for (int i = 0; i < group.attributeNumber; i++)
			doOverlap = doOverlap && doOverlap(group, i);
		
		return doOverlap;
	}

	public boolean doOverlap(TuplesGroup1 group, int attributeID)
	{
		IntAttributeRange attrRange1 = group.getAttributeRange(attributeID);
		IntAttributeRange attrRange2 = this.getAttributeRange(attributeID);
		
		IntAttributeRange intersection = attrRange1.intersect(attrRange2);
		
		return (intersection.isEmpty() == false);
	}

	public boolean doOverlap(Tuple tuple)
	{
		for (int i = 0; i < tuple.getAttributesNumber(); i++)
			if (!doOverlap(tuple, i))
				return false;
		
		return true;
	}
	
	// Reduce groups with all attributes having the same value (sites can be different).
	public void reduceRedundantGroups() {
		
		// reduce groups until you can.
		while (reduceRedundantGroupsIfPossible());
		
	}
	
	private boolean reduceRedundantGroupsIfPossible() {

		if (this.hasSubGroups())
		{
			if (this.getLeftChild().hasSubGroups() && this.getRightChild().hasSubGroups()) 
			{
				this.getLeftChild().reduceRedundantGroups();
				this.getRightChild().reduceRedundantGroups();
			}
			else		
			{
				IntAttributeRange[] leftAttributes = this.getLeftChild().getAttributeRanges();
				IntAttributeRange[] rightAttributes = this.getRightChild().getAttributeRanges();
				
				boolean areEqual = true;
				if (leftAttributes.length == rightAttributes.length) {
					for (int i = 0; i < leftAttributes.length && areEqual; i++)
						areEqual = leftAttributes[i].isEqualTo(rightAttributes[i]);
				}
				
				if (areEqual) {
					this.setTuples(this.getTuples());
					return true;
				}
			}
		}
		
		return false;
	}
	
	public boolean doOverlap(Tuple tuple, int attributeID)
	{
		int attr1 = tuple.getData(attributeID);
		IntAttributeRange attrRange2 = this.getAttributeRange(attributeID);

		return (attrRange2.isWithin(attr1));
	}
	
	public TuplesGroup1[] split(int attribute, double value, int minNumTuples, Site site) {
		List<Tuple> lGroup = new ArrayList<Tuple>();
		List<Tuple> rGroup = new ArrayList<Tuple>();
		List<Tuple> mGroup = new ArrayList<Tuple>();
		for (int i = 0; i < data.size(); i++) {
			int rawData = data.get(i).getData()[attribute];
			
			if (site != null && data.get(i).getSite() != site)
				rGroup.add(data.get(i));

			if (rawData < value) {
				lGroup.add(data.get(i));
			} else if (rawData > value) {
				rGroup.add(data.get(i));
			} else {
				mGroup.add(data.get(i));
			}
		}
		
		if (lGroup.size() < minNumTuples && rGroup.size() < minNumTuples)
		{
			int splitIndex = mGroup.size() - (minNumTuples - lGroup.size() + minNumTuples - rGroup.size());
			splitIndex /= 2;
			splitIndex += (minNumTuples - lGroup.size());
			splitIndex = Math.max(0, splitIndex);
			lGroup.addAll(mGroup.subList(0, splitIndex));
			rGroup.addAll(mGroup.subList(splitIndex, mGroup.size()));
		} 
		else if (lGroup.size() < minNumTuples)
			lGroup.addAll(mGroup);
		else
			rGroup.addAll(mGroup);
			
		return new TuplesGroup1[] {new TuplesGroup1(lGroup.toArray(new Tuple[] {})), new TuplesGroup1(rGroup.toArray(new Tuple[] {})this.getStats())};
	}

	public TuplesGroup1 splitToOneGroup(int attribute, double value, int minNumTuples, Site site) {
		List<Tuple> lGroup = new ArrayList<Tuple>();
		List<Tuple> rGroup = new ArrayList<Tuple>();
		List<Tuple> mGroup = new ArrayList<Tuple>();
		for (int i = 0; i < data.size(); i++) {
			int rawData = data.get(i).getData()[attribute];
			
			if (site != null && data.get(i).getSite() != site)
				rGroup.add(data.get(i));
			
			if (rawData < value) {
				lGroup.add(data.get(i));
			} else if (rawData > value) {
				rGroup.add(data.get(i));
			} else {
				mGroup.add(data.get(i));
			}
		}
		
		if (lGroup.size() < minNumTuples && rGroup.size() < minNumTuples)
		{
			int splitIndex = minNumTuples - (minNumTuples - lGroup.size() + minNumTuples - rGroup.size());
			splitIndex /= 2;
			splitIndex += (minNumTuples - lGroup.size());
			lGroup.addAll(mGroup.subList(0, splitIndex));
			rGroup.addAll(mGroup.subList(splitIndex, mGroup.size()));
		} 
		else if (lGroup.size() < minNumTuples)
			lGroup.addAll(mGroup);
		else
			rGroup.addAll(mGroup);
		
		TuplesGroup1 group = new TuplesGroup1((Tuple [])null);
		group.setTuplesGroup1s(new TuplesGroup1(lGroup.toArray(new Tuple[] {})), new TuplesGroup1(rGroup.toArray(new Tuple[] {})));
		
		return group;
	}
	
	public Set<Site> getSites() {
		return this.sites;
	}
	
	public void setTuplesGroup1s(TuplesGroup1 left, TuplesGroup1 right) {
		this.sites = new HashSet<Site>();

		this.left = left;
		if (this.left != null) {
			this.left.parent = this;
			this.sites.addAll(left.getSites());
			}
		this.right = right;
		if (this.right != null) {
			this.right.parent = this;
			this.sites.addAll(right.getSites());
		}
		this.data = null;
		
		this.attributeNumber = Math.max(left == null ? 0 : left.getNumAttributes(), right == null ? 0 : right.getNumAttributes());
		this.attributeRanges = computeAttributeRanges();
	}
	
	public Map<Integer, Integer> getAttributeFrequenciesUncorrupted(int attributeID) {
		return Tools.getUncorruptedFrequencies(this, attributeID);
	}

	public Map<Integer, Double> getAttributeValueProbabilities(int attributeID, boolean onlyUncorrupted) {
		return Tools.getValueProbabilities(this, attributeID, onlyUncorrupted);
	}

	public void clearAttributeRanges()
	{
		this.attributeRanges = null;
	}
	
	private IntAttributeRange[] computeAttributeRanges() {
		
		if (this.hasSubGroups() == true)
		{
			IntAttributeRange[] leftAttributeRanges = this.getLeftChild().getAttributeRanges();
			IntAttributeRange[] rightAttributeRanges = this.getRightChild().getAttributeRanges();
			
			if (leftAttributeRanges == null || rightAttributeRanges == null)
				return null; 
			
			IntAttributeRange[] ranges = new IntAttributeRange[leftAttributeRanges.length];
			for (int i = 0; i < ranges.length; i++)
				ranges[i] = new IntAttributeRange(
					Math.min(leftAttributeRanges[i].getMin(), rightAttributeRanges[i].getMin()),
					Math.max(leftAttributeRanges[i].getMax(), rightAttributeRanges[i].getMax())
				);

			return ranges;
		}
		
		if (this.data == null || this.data.size() == 0)
			return null;
				
		int numQIDs = this.getNumAttributes();
		int[][] minmax = new int[numQIDs][2];
		int[] rawData = data.get(0).getData();
		for (int j = 0; j < numQIDs; j++) {
			minmax[j][0] = Integer.MAX_VALUE;
			minmax[j][1] = Integer.MIN_VALUE;
		}
		for (int i = 0; i < data.size(); i++) {
			if (data.get(i).isCorrupted())
				continue;
			rawData = data.get(i).getData();
			for (int j = 0; j < numQIDs; j++) {
				if (minmax[j][0] > rawData[j]) {
					minmax[j][0] = rawData[j];
				}
				if (minmax[j][1] < rawData[j]) {
					minmax[j][1] = rawData[j];
				}
			}
		}

		IntAttributeRange[] range = new IntAttributeRange[numQIDs];
		for (int i = 0; i < range.length; i++) {
			range[i] = new IntAttributeRange(minmax[i][0], minmax[i][1]);
		}
		return range;
	}
	
	public IntAttributeRange[] getAttributeRanges() {
		if (attributeRanges == null)
			this.attributeRanges = computeAttributeRanges();
		
		return this.attributeRanges;
	}
	
	public String getStringRanges() {
		StringBuffer b = new StringBuffer();
		for (int i = 0; i < this.getAttributeRanges().length; i++) {
			b.append("[").append(this.getAttributeRange(i).getMin()).append("-").append(this.getAttributeRange(i).getMax()).append("]");
		}
		return b.toString();
	}

	public void printRangesAndTuples(PrintStream str) {
		if (this.hasSubGroups()) {
			left.printRangesAndTuples(str);
			right.printRangesAndTuples(str);
		} else {
			for (int i = 0; i < data.size(); i++) {
				str.print(getStringRanges());
				str.print(",");
				str.print(data.get(i));
				str.print(",");
				str.print(data.get(i).getSiteID());
				str.println();
			}
		}
	}
	
	
//	 * File format:
//	 * - [<group ID>][L][<left child group ID>]
//	 * - [<group ID>][R][<right child group ID>]
//	 * - [<group ID>]<attributes>
//	 * @param str
	 
	public void printAllGroups(PrintStream str) {
		if (this.hasSubGroups()) {
			str.println("[" + this.getGroupID() + "][" + TuplesGroup1.LEFT_CHILD + "][" + left.getGroupID() + "]");
			left.printAllGroups(str);
			str.println("[" + this.getGroupID() + "][" + TuplesGroup1.RIGHT_CHILD + "][" + right.getGroupID() + "]");
			right.printAllGroups(str);
		} else {
			str.println("[" + this.getGroupID() + "][" + TuplesGroup1.DATA + "]" + getStringRanges());
		}
	}
	
	private int getGroupID()
	{
		return this.groupID;
	}

	public void printGroups(PrintStream str, boolean withStats) {
		if (this.hasSubGroups()) {
			left.printGroups(str, withStats);
			right.printGroups(str, withStats);
		} else {
			str.print(getStringRanges());
			if (withStats) {
				str.print(",");
				str.print(this.data.size());
				str.print(",");
				str.print(this.getNumUncorruptedTuples());
	
				Map<Integer, Integer> site2freq = this.getAttributeFrequenciesUncorrupted(TuplesGroup1.SENSITIVE);
				for (Integer siteID : site2freq.keySet())
					str.print("," + siteID + "(" +site2freq.get(siteID)+ ")");
			}
			str.println();
		}
	}
	
	public int getCountOfDistinctNodes(boolean onlyUncorrupted) {
		int[] hist = getNodesHistogram(onlyUncorrupted);
		int s = 0;
		for (int i = 0; i < hist.length; i++) {
			if (hist[i] != 0) {
				s++;
			}
		}
		return s;
	}
	
	public double getStdDeviationForNodesDistribution() {
		int[] histogram = getNodesHistogram(false);
		double avg = 0;
		for (int i = 0; i < histogram.length; i++) {
			avg += histogram[i];
		}
		avg = avg/(double)histogram.length;
		double sum = 0;
		for (int i = 0; i < histogram.length; i++) {
			sum += (avg - histogram[i])*(avg - histogram[i])/(double)(histogram.length - 1);
		}
		return Math.sqrt(sum);
	}

	public int[] getNodesHistogram(boolean onlyUncorrupted) {
		int hist[] = new int[Main.CURR_N];
		if (!this.hasSubGroups()) {
			for (int i = 0; i < data.size(); i++) {
				
				if (onlyUncorrupted && data.get(i).isCorrupted())
					continue;
				
				hist[data.get(i).getSiteID()]++;
			}
		} else {
			int[] histLeft = left.getNodesHistogram(onlyUncorrupted);
			int[] histRight = right.getNodesHistogram(onlyUncorrupted);
			for (int i = 0; i < hist.length; i++) {
				hist[i] = histLeft[i] + histRight[i];
			}
		}
		return hist;
	}

	public Tuple[] getData() {
		return data.toArray(new Tuple[0]);
	}

	public TuplesGroup1 getLeftChild() {
		return left;
	}

	public TuplesGroup1 getRightChild() {
		return right;
	}
	
	public TuplesGroup1 getParent() {
		return this.parent;
	}

	public void printHistogramOfNodes() {
		int[] hist = getNodesHistogram(false);
		for (int i = 0; i < hist.length; i++) {
			System.out.println("\"Node " + i + "\"," + hist[i]);
		}
	}
	
	public double getAvgGroupSize(boolean onlyUncorrupted) {
		Map<String, Num> groups = new HashMap<String, Num>();
		getGroups(groups, onlyUncorrupted);
		int sum = 0;
		for (Iterator<Num> iterator = groups.values().iterator(); iterator.hasNext();) {
			Num n = (Num) iterator.next();
			sum += n.cnt;
		}
		return sum / (double)groups.size();
	}

	public double getAvgNumOfNodesInGroup(boolean onlyUncorrupted) {
		int sum = sumDistinctNodes(onlyUncorrupted);
		int cnt = countGroups(false);
		return sum / (double)cnt;
	}

	private int sumDistinctNodes(boolean onlyUncorrupted) {
		if (!this.hasSubGroups()) {
			return getCountOfDistinctNodes(onlyUncorrupted);
		} else {
			return left.sumDistinctNodes(onlyUncorrupted) + right.sumDistinctNodes(onlyUncorrupted);
		}
	}
	
	public int countGroups(final boolean onlyUncorrupted) {
		if (this.hasSubGroups() == false) {
			
			if (onlyUncorrupted == false)
				return 1;
			
			int i = 0;
			while (i < this.getNumTuples() && this.getTuples()[i].isCorrupted() == true)
				i++;
			
			return (i == this.getNumTuples() ? 0 : 1);
			
		} else {
			return left.countGroups(onlyUncorrupted) + right.countGroups(onlyUncorrupted);
		}
	}
	
	private void getGroups(Map<String, Num> map, boolean onlyUncorrupted) {
		if (this.hasSubGroups() == false) {
			String grp = this.getStringRanges();
			Num num = (Num) map.get(grp);
			if (num == null) {
				num = new Num();
				map.put(grp, num);
			}
			
			for (Tuple t : data)
			{
				if (onlyUncorrupted && t.isCorrupted() == true)
					continue;
				
				num.cnt++;
			}
			
		} else {
			left.getGroups(map, onlyUncorrupted);
			right.getGroups(map, onlyUncorrupted);
		}
	}
	
	public void printNumberOfOwnersDistribution(PrintStream ps)
	{
		Map<String, Map<Integer, Integer> > map = new HashMap<String, Map<Integer, Integer> >();
		this.getNodesDistribution(map);
		
		for (String s : map.keySet())
		{
			ps.print(s + ": " + map.get(s).keySet().size());
			ps.println();
		}
	}
	
	public  void printNodesDistribution(PrintStream ps) {
		Map<String, Map<Integer, Integer> > map = new HashMap<String, Map<Integer, Integer> >();
		this.getNodesDistribution(map);
		Map<Integer, Integer> o;
		
		for (String s : map.keySet())
		{
			ps.print(s + ": ");
			o = map.get(s);
			for (Integer groupID : o.keySet())
				ps.print(groupID + "(" + o.get(groupID) + "), ");
			ps.println();
		}
	}
	
	public void getNodesDistribution(Map<String, Map<Integer, Integer> > map) {
		if (!this.hasSubGroups()) {
			String grp = this.getStringRanges();
			Map<Integer, Integer> group2freq = map.get(grp);
			if (group2freq == null)
				group2freq = new TreeMap<Integer, Integer>();

			for (int i = 0; i < this.data.size(); i++) {
				Integer currentOwner = this.data.get(i).getSiteID();
				Integer currentFreq = group2freq.get(currentOwner);
				if (currentFreq == null)
					currentFreq = 0;
				currentFreq++;
				group2freq.put(currentOwner, currentFreq);
			}

			map.put(grp, group2freq);
			
		} else {
			left.getNodesDistribution(map);
			right.getNodesDistribution(map);
		}
	}

	public double getSumOfStdDeviationForNodesDistribution() {
		Map<String, Map<Integer, Integer> > distribution = new TreeMap<String, Map<Integer, Integer> > ();
		getNodesDistribution(distribution);
		double totalSum = 0.0;
	
		// for each group
		for (Map<Integer, Integer> groupStats : distribution.values())
		{
			double avg = 0.0;
			double sum = 0.0;
			
			for (Integer freq : groupStats.values())
				avg += freq;
			
			avg /= (double)groupStats.values().size();		
			
			for (Integer freq : groupStats.values())
				sum += Math.abs((int)(avg - freq));//*(avg - freq);
			
			totalSum += sum;//Math.sqrt(sum / (double)(groupStats.values().size() - 1));
		}

		return totalSum / (double) this.countGroups(false);
	}

	
	private class Num {
		int cnt = 0;
	}


	public int getNumTuples() {
		if (hasSubGroups())
			return this.getLeftChild().getNumTuples() + this.getRightChild().getNumTuples();
		else
			return this.data.size();
	}
	
	public Tuple[] getTuples()
	{
		return this.getTuplesAsList().toArray(new Tuple[0]);
	}
	
	public List<Tuple> getTuplesAsList()
	{
		if (hasSubGroups())
		{
			List<Tuple> allTuples = new ArrayList<Tuple>();
			
			allTuples.addAll(Arrays.asList(this.getLeftChild().getTuples()));
			allTuples.addAll(Arrays.asList(this.getRightChild().getTuples()));
			
			return allTuples;
		}
		else
			return this.data;
	}

	public boolean hasSubGroups() {
		return (this.left != null && this.right != null);
	}

//	public void removeTuples(Tuple[] corruptedTuples) {
//		
//		if (hasSubGroups())
//		{
//			this.getLeftGroup().removeTuples(corruptedTuples);
//			this.getRightGroup().removeTuples(corruptedTuples);
//			
//			// check if there are any tuples, if not - reduce tree of tuple groups.
//			if (this.getLeftGroup().getNumTuples() == 0)
//			{
//				this.data = this.getRightGroup().getData();
//				this.attributeRanges = this.getRightGroup().getAttributeRanges();
//				// update attribute ranges
//				for (int i = 0; i < this.attributeRanges.length; i++)
//				{
//					this.attributeRanges[i].setMinMax(
//							Math.min(this.getLeftGroup().getAttributeRanges()[i].getMin(), this.getRightGroup().getAttributeRanges()[i].getMin()), 
//							Math.max(this.getLeftGroup().getAttributeRanges()[i].getMax(), this.getRightGroup().getAttributeRanges()[i].getMax())
//					);
//				}
//				
//				this.left = this.getRightGroup().getLeftGroup();
//				this.right = this.getRightGroup().getRightGroup();
//			}
//			else if (this.getRightGroup().getNumTuples() == 0)
//			{
//				this.data = this.getLeftGroup().getData();
//				this.attributeRanges = this.getLeftGroup().getAttributeRanges();
//
//				// update attribute ranges
//				for (int i = 0; i < this.attributeRanges.length; i++)
//				{
//					this.attributeRanges[i].setMinMax(
//							Math.min(this.getLeftGroup().getAttributeRanges()[i].getMin(), this.getRightGroup().getAttributeRanges()[i].getMin()), 
//							Math.max(this.getLeftGroup().getAttributeRanges()[i].getMax(), this.getRightGroup().getAttributeRanges()[i].getMax())
//					);
//				}
//
//				this.right = this.getLeftGroup().getRightGroup();
//				this.left = this.getLeftGroup().getLeftGroup();
//			}
//		}
//		else
//		{
//			List<Tuple> corruptedList = Arrays.asList(corruptedTuples);
//			ArrayList<Tuple> secureTuples = new ArrayList<Tuple>();
//			for (Tuple tuple : this.getData())
//			{
//				if (corruptedList.contains(tuple) == false)
//					secureTuples.add(tuple);
//			}
//			
//			if (secureTuples.size() != this.getData().length)
//			{
//				this.data = secureTuples.toArray(new Tuple[0]);
//				
//				// let's keep attribute ranges intact
//				//this.attributeRanges = this.computeAttributeRanges();
//			}
//		}
//
//	}
	
//	public Tuple[] getTuplesFromOwner(int ownerID)
//	{
//		ArrayList<Tuple> ownedTuples = new ArrayList<Tuple>();
//		
//		for (Tuple tuple : this.getTuples())
//		{
//			
//			if (tuple.getSiteID() == ownerID)
//				ownedTuples.add(tuple);
//		}
//		
//		return ownedTuples.toArray(new Tuple[0]);
//	}

	public int getNumAttributes() {
		
		if (this.hasSubGroups())
			return this.getLeftChild().getNumAttributes();
		
		if (this.data == null || this.data.size() == 0)
		{
			this.attributeNumber = 0;
			return this.attributeNumber;
		}
		
		this.attributeNumber = this.data.get(0).getAttributesNumber();
		
		return this.attributeNumber;
	}

	public IntAttributeRange getAttributeRange(int i) {
		return this.getAttributeRanges()[i];
	}

	public int getNumUncorruptedTuples() {
		if (hasSubGroups())
			return this.getLeftChild().getNumUncorruptedTuples() + this.getRightChild().getNumUncorruptedTuples();
		else
		{
			int counter = 0;
			for (Tuple t : this.data)
			{
				if (t.isCorrupted() == false)
					counter++;
			}
			
			return counter;
		}
	}

	public Tuple[] getUncorruptedTuples() {
		return this.getUncorruptedTuplesAsList().toArray(new Tuple[0]);
	}
	
	public List<Tuple> getUncorruptedTuplesAsList() {
	
		if (hasSubGroups())
		{
			List<Tuple> allTuples = new ArrayList<Tuple>();
			
			allTuples.addAll(Arrays.asList(this.getLeftChild().getTuples()));
			allTuples.addAll(Arrays.asList(this.getRightChild().getTuples()));
			
			return allTuples;
		}
		else
		{
			ArrayList<Tuple> uncorruptedTuples = new ArrayList<Tuple>();
			
			for (Tuple t: this.data)
				if (t.isCorrupted() == false)
					uncorruptedTuples.add(t);
			
			return uncorruptedTuples;
		}
				
	}

	public void clearTuples()
	{
		this.data = null;
		this.sites.clear();
		if (this.getLeftChild() != null)
			this.sites.addAll(this.getLeftChild().getSites());
		if (this.getRightChild() != null)
			this.sites.addAll(this.getRightChild().getSites());
		this.clearAttributeRanges();
	}
	
	public void setTuples(Tuple[] ts) {
		this.left = null;
		this.right = null;
		this.data = new ArrayList<Tuple>(ts.length);
		this.data.addAll(Arrays.asList(ts));
		this.sites = new HashSet<Site>();
		for (Tuple t : ts)
			this.sites.add(t.getSite());
		
		this.attributeNumber = (data == null || data.size() == 0 ? 0 : this.data.get(0).getData().length - 1);
		this.attributeRanges = computeAttributeRanges();
	}
	
	public Map<Site, Integer> getSiteFrequencies() {
		return Tools.getSitesFrequencies(this);
	}

	public Map<Site, List<Tuple>> getTuplesSplittedToSites() {
		
		Map<Site, List<Tuple>> sites2tuples = new HashMap<Site, List<Tuple>>();
		
		for (Tuple t : this.getTuples())
		{
			if (t.isCorrupted())
				continue;
			
			Site site = t.getSite();
			List<Tuple> tuples = sites2tuples.get(site);
			
			if (tuples == null)
			{
				tuples = new ArrayList<Tuple>();
				sites2tuples.put(site, tuples);
				tuples = sites2tuples.get(site);
			}
			
			tuples.add(t);
		}
		
		return sites2tuples;
	}

	public Set<Site> getRepresentedSites()
	{
		Set<Site> sites = new HashSet<Site>();
		
		for (Tuple t : this.getTuples())
		{
			if (t.isCorrupted())
				continue;
			
			sites.add(t.getSite());
		}
		
		return sites;
	}
	
	public int getNumRepresentedSites() {
		return this.getRepresentedSites().size();
	}

	// get corrupted and uncorrupted tuples
	public List<Tuple> getTuplesFromSite(Site site) {
		
		ArrayList<Tuple> ownedTuples = new ArrayList<Tuple>();
		
		for (Tuple tuple : this.getTuples())
		{
			if (tuple.getSite().equals(site))
				ownedTuples.add(tuple);
		}
		
		return ownedTuples;
	}

	public void addTuple(Tuple tuple)
	{
		this.data.add(tuple);
		this.sites.add(tuple.getSite());
//		this.stats.addTupleIfNeeded(tuple);
		if (this.attributeRanges != null)
			for (int i = 0; i < this.attributeRanges.length; i++)
				this.attributeRanges[i].updateFor(tuple.getData(i));
	}
	
	public void addTuples(List<Tuple> tuples)
	{
		if (this.data == null) {
			this.data = new ArrayList<Tuple>(tuples.size());
			this.sites = new HashSet<Site>();
		}
		this.data.addAll(tuples);
		for (Tuple t : tuples){
			this.sites.add(t.getSite());
//			this.stats.addTupleIfNeeded(t);
			if (this.attributeRanges != null)
				for (int i = 0; i < this.attributeRanges.length; i++)
					this.attributeRanges[i].updateFor(t.getData(i));
		}
	}

	public List<TuplesGroup1> getIntersectingLeafGroups(TuplesGroup1 tuples)
	{
		if (this.hasSubGroups())
		{
			List<TuplesGroup1> leftGroups = this.getLeftChild().getIntersectingLeafGroups(tuples);
			List<TuplesGroup1> rightGroups = this.getRightChild().getIntersectingLeafGroups(tuples);
			
			List<TuplesGroup1> tg = new ArrayList<TuplesGroup1>(leftGroups.size() + rightGroups.size());
			tg.addAll(leftGroups);
			tg.addAll(rightGroups);

			return tg;			
		}
		else
		{
			List<TuplesGroup1> tg = new ArrayList<TuplesGroup1>(1);
			if (this.doOverlap(tuples))
				tg.add(this);
			
			return tg;
		}
	}

//	 * - [<group ID>][L][<left child group ID>]
//	 * - [<group ID>][R][<right child group ID>]
//	 * - [<group ID>][D]<attributes>
//	 * @param fileName
//	 * @return
//	 * @throws NumberFormatException
//	 * @throws IOException
	public static TuplesGroup1 readGroups(String fileName) throws NumberFormatException, IOException
	{
		BufferedReader reader = new BufferedReader(new FileReader(fileName));
		String line;
		Map<Integer, TuplesGroup1> ids2Groups = new HashMap<Integer, TuplesGroup1>();
		while ((line = reader.readLine()) != null) {
			String[] attrs = line.replaceAll(" ", "").split("\\]\\[");
			String lastAttr = attrs[attrs.length - 1];
			attrs[attrs.length - 1] = lastAttr.substring(0, lastAttr.length() - 1);
			int thisID = Integer.parseInt(attrs[0].substring(1));

			if (!attrs[1].equals(TuplesGroup1.DATA)) {
	
				TuplesGroup1 parent = ids2Groups.get(thisID);
				if (parent == null){
					parent = new TuplesGroup1(null, null);
					ids2Groups.put(thisID, parent);	
				}
				int childID = Integer.parseInt(attrs[2]);
				TuplesGroup1 child = ids2Groups.get(childID);
				if (child == null) {
					child = new TuplesGroup1(null, null);
					ids2Groups.put(childID, child);
				}
				child.parent = parent;
				
				if (attrs[1].equals(TuplesGroup1.RIGHT_CHILD)) {
					parent.setTuplesGroup1s(parent.getLeftChild(), child);
				}
				else if (attrs[1].equals(TuplesGroup1.LEFT_CHILD))
					parent.setTuplesGroup1s(child, parent.getRightChild());
			}
			else
			{
				IntAttributeRange[] vals = new IntAttributeRange[attrs.length - 2];
				for (int i = 2; i < attrs.length; i++) {
					vals[i-2] = new IntAttributeRange(attrs[i]);
				}
				TuplesGroup1 leaf = ids2Groups.get(thisID);
				if (leaf == null)
				{
					leaf = new TuplesGroup1(null, vals);
					ids2Groups.put(thisID, leaf);
				}
				else {
					leaf.attributeRanges = vals;
				}
			}
		}
		reader.close();
		
		return findRoot(ids2Groups.values());
	}

	public static TuplesGroup1 findRoot(Collection<TuplesGroup1> values)
	{
		Iterator<TuplesGroup1> it = values.iterator();
		TuplesGroup1 root = it.next();
		while (root.getParent() != null && it.hasNext())
			root = it.next();

		if (!it.hasNext() && root.getParent() != null)
			return null;
		
		return root;
	}

	public boolean canContainTuple(Tuple t)
	{
		int i = 0;
		while (i < t.getAttributesNumber() && this.attributeRanges[i].isWithin(t.getData(i)))
			i++;
		
		return i == t.getAttributesNumber();
	}

*/
}
