package edu.emory.mathcs.anonymizers;

import edu.emory.mathcs.Tools;
import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.privacy.AndCheckers;
import edu.emory.mathcs.privacy.K_anonymity;
import edu.emory.mathcs.privacy.L_diversity;
import edu.emory.mathcs.privacy.PrivacyChecker;


public class KLDummySplitter implements Anonymizer {

	private int k;
	private int l;
	
	public KLDummySplitter(int k, int l) {
		this.k = k;
		this.l = l;
	}
	
	public void splitGroup(TuplesGroup group) {
		
		if (group.getNumTuples() < 2 * k) {
			return;
		}
		
		int[] rangeLen = new int[group.getNumAttributes()];
		rangeLen[0] = group.getAttributeRange(0).getMax() - group.getAttributeRange(0).getMin();
		int maxRange = rangeLen[0];
		int minRange = maxRange;
		for (int i = 1; i < rangeLen.length; i++) {
			rangeLen[i] = group.getAttributeRange(i).getMax() - group.getAttributeRange(i).getMin();
			if (rangeLen[i] > maxRange) {
				maxRange = rangeLen[i];
			}
			if (rangeLen[i] < minRange) {
				minRange = rangeLen[i];
			}
		}
		
		TuplesGroup[][] groups = new TuplesGroup[group.getNumAttributes()][];
		for (int i = 0; i < groups.length; i++) {
			groups[i] = group.split(i, Tools.findMedian(i, group.getData()), this.k, null);
		}
		
		int best = -1;
		double bestScore = -1;
		for (int i = 0; i < group.getNumAttributes(); i++) {
			if (groups[i][0].getCountOfDistinctNodes(false) < l || groups[i][1].getCountOfDistinctNodes(false) < l) {
				continue;
			}
			double score = rangeLen[i];
			if (score > bestScore) {
				bestScore = score;
				best = i;
			}
		}
		
		if (best != -1) {
			group.setTuplesGroups(groups[best][0], groups[best][1]);
			splitGroup(groups[best][0]);
			splitGroup(groups[best][1]);
		}
		
	}

	public String getOutFileName() {
		return "Dummy";
	}
	
	@Override
	public PrivacyChecker getPrivacyChecker()
	{
		return new AndCheckers.AndCheckersBuilder()
			.addChecker(new L_diversity(this.l), 1.0)
			.addChecker(new K_anonymity(this.k), 1.0)
			.build("dummy_splitter_checker");
	}

}
