package edu.emory.mathcs.anonymizers;

import edu.emory.mathcs.Tools;
import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.privacy.PrivacyChecker;

public class MondrianSplitter implements Anonymizer
{
	private PrivacyChecker checker;

	public MondrianSplitter(PrivacyChecker checker)
	{
		this.checker = checker;
	}

	public void splitGroup(TuplesGroup group)
	{
		// Medians and attribute ranges
		double[] median = new double[group.getNumAttributes()];
		for (int i = 0; i < median.length; i++)
		{
			median[i] = Tools.findMedian(i, group.getData());
		}
		int[] rangeLen = new int[median.length];
		rangeLen[0] = group.getAttributeRange(0).getMax() - group.getAttributeRange(0).getMin();
		int maxRange = rangeLen[0];
		int minRange = maxRange;
		for (int i = 1; i < rangeLen.length; i++)
		{
			rangeLen[i] = group.getAttributeRange(i).getMax() - group.getAttributeRange(i).getMin();
			if (rangeLen[i] > maxRange)
			{
				maxRange = rangeLen[i];
			}
			if (rangeLen[i] < minRange)
			{
				minRange = rangeLen[i];
			}
		}

		// Possible splits
		TuplesGroup[][] groups = new TuplesGroup[median.length][];
		// Now I have rangesLen, maxRange, minNodes, min, max
		int best = -1;
		double bestScore = -1;
		for (int i = 0; i < median.length; i++)
		{
			groups[i] = group.split(i, median[i], 1, null);
			if (this.checker.isAnonymized(groups[i][0]) == false || this.checker.isAnonymized(groups[i][1]) == false)
				continue;

			double score = Tools.getProjectedTo01(minRange, maxRange, rangeLen[i]);

			if (score > bestScore)
			{
				bestScore = score;
				best = i;
			}
		}

		if (best != -1)
		{
			group.setTuplesGroups(groups[best][0], groups[best][1]);
			splitGroup(groups[best][0]);
			splitGroup(groups[best][1]);
		}

	}

	public String getOutFileName()
	{
		return "Mondrian";
	}

	@Override
	public PrivacyChecker getPrivacyChecker()
	{
		return this.checker;
	}
}
