package edu.emory.mathcs.anonymizers;

import edu.emory.mathcs.Tools;
import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.privacy.K_anonymity;
import edu.emory.mathcs.privacy.PrivacyChecker;


public class StandardSplitter implements Anonymizer {

	private int k;
	
	public StandardSplitter(int k) {
		this.k = k;
	}
	
	public void splitGroup(TuplesGroup group) {
		
		if (group.getNumTuples() < 2 * k) {
			return;
		}
		
		int maxSpread = 0;
		int bestAttr = 0;
		for (int i = 0; i < group.getNumAttributes(); i++) {
			if (maxSpread < group.getAttributeRange(i).getMax() - group.getAttributeRange(i).getMin()) {
				maxSpread = group.getAttributeRange(i).getMax() - group.getAttributeRange(i).getMin();
				bestAttr = i;
			}
		}
		double median = Tools.findMedian(bestAttr, group.getData());
		TuplesGroup g[] = group.split(bestAttr, median, this.k, null);
		
		group.setTuplesGroups(g[0], g[1]);
		
		splitGroup(g[0]);
		splitGroup(g[1]);
	}

	public String getOutFileName() {
		return "Standard";
	}

	@Override
	public PrivacyChecker getPrivacyChecker()
	{
		return new K_anonymity(this.k);
	}
}
