package edu.emory.mathcs.measures;

import java.util.Map;
import java.util.TreeMap;

import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;

public class LinearCombinationMeasure implements Measure 
{
	private Map<Measure, Double> components;
	private String name;
	
	public static class LinearCombinationMeasureBuilder 
	{
//		private double minRange;
//		private double maxRange;
		private Map<Measure, Double> components;

		public LinearCombinationMeasureBuilder()
		{
//			this.minRange = 0.0;
//			this.maxRange = 1.0;
			this.components = new TreeMap<Measure, Double>();
		}
		
//		public LinearCombinationMeasureBuilder(double minRange, double maxRange)
//		{
//			this.minRange = minRange;
//			this.maxRange = maxRange;
//			this.components = new Hashtable<Measure, Double>();
//		}
		
		public LinearCombinationMeasureBuilder addMeasure(Measure m, double weight)
		{
			this.components.put(m, weight);
			return this;
		}

		public LinearCombinationMeasureBuilder removeMeasure(Measure m, double weight)
		{
			this.components.remove(m);
			return this;
		}

		public LinearCombinationMeasure build(String name)
		{
//			double sum = 0.0;
//			for (Double w : this.components.values())
//				sum += w;

//			double rangeLen = this.maxRange - this. minRange;
//			for (Measure m : this.components.keySet())
//			{
//				double w = this.components.get(m);
//				this.components.put(m, this.minRange + rangeLen * w /*/ sum*/);
//			}
//			
			return new LinearCombinationMeasure(name, this.components);
		}
		
	}
	
	public LinearCombinationMeasure(String name, Map<Measure, Double> components)
	{
		this.name = name;
		this.components = components;
	}
	
	@Override
	public double getMaxValue(final Tuple[] tuples) {
		double weightedValue = 0.0;
		for (Measure m : this.components.keySet())
			weightedValue += this.components.get(m) * m.getMaxValue(tuples);
		
		return weightedValue;
	}

	@Override
	public double getMinValue(final Tuple[] tuples) {
		double weightedValue = 0.0;
		for (Measure m : this.components.keySet())
			weightedValue += this.components.get(m) * m.getMinValue(tuples);
		
		return weightedValue;
	}
//
//	@Override
//	public double getPossibleMaxValue() {
//		double weightedValue = 0.0;
//		for (Measure m : this.components.keySet())
//			weightedValue += this.components.get(m) * m.getPossibleMaxValue();
//		
//		return weightedValue;
//	}
//
//	@Override
//	public double getPossibleMaxValue(int numTuples) {
//		double weightedValue = 0.0;
//		for (Measure m : this.components.keySet())
//			weightedValue += this.components.get(m) * m.getPossibleMaxValue(numTuples);
//		
//		return weightedValue;
//	}
//
//	@Override
//	public double getPossibleMinValue() {
//		double weightedValue = 0.0;
//		for (Measure m : this.components.keySet())
//			weightedValue += this.components.get(m) * m.getPossibleMinValue();
//		
//		return weightedValue;
//	}
//
//	@Override
//	public double getPossibleMinValue(int numTuples) {
//		double weightedValue = 0.0;
//		
//		for (Measure m : this.components.keySet())
//			weightedValue += this.components.get(m) * m.getPossibleMinValue(numTuples);
//		
//		return weightedValue;
//	}

	@Override
	public double getValue(TuplesGroup tuples) {
	
		double weightedValue = 0.0;
		
		for (Measure m : this.components.keySet())
			weightedValue += this.components.get(m) * m.getValue(tuples);
		
		return weightedValue;
	}

	@Override
	public String getName() {
		return this.name;
	}
	
	@Override
	public int compareTo(Measure o) {
		return o.getName().compareTo(this.getName());
	}

	@Override
	public double getBetterScore(double score1, double score2) {
		return Math.max(score1, score2);
	}

	@Override
	public double getCombinedMeasureValue(double leftValue, double rightValue) {
		return Double.NaN;
	}

}
