package edu.emory.mathcs.measures;

import java.util.Set;
import java.util.TreeSet;

import edu.emory.mathcs.Site;
import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;

public class MinCountSites implements Measure
{

	public static final String NAME = "site l-diverse";

	@Override
	public double getBetterScore(double score1, double score2)
	{
		return Math.max(score1, score2);
	}

	@Override
	public double getMaxValue(Tuple[] tuples)
	{
		if (tuples.length == 0)
			return Double.NaN;
		
		Set<Site> sites = new TreeSet<Site>();
		
		for (Tuple tuple : tuples)
			sites.add(tuple.getSite());
		
		return sites.size();
	}

	@Override
	public double getMinValue(Tuple[] tuples)
	{
		if (tuples.length == 0)
			return Double.NaN;
		
		return 0.0;
	}

	@Override
	public String getName()
	{
		return MinCountSites.NAME;
	}

	@Override
	public double getValue(TuplesGroup tuples)
	{
		double minNumSites = 0.0;

		if (tuples.hasSubGroups())
		{
			minNumSites = this.getCombinedMeasureValue(this.getValue(tuples.getLeftChild()), this.getValue(tuples.getRightChild()));
		}
		else
		{
			minNumSites = tuples.getNumRepresentedSites();
		}

		return minNumSites;
	}

	@Override
	public int compareTo(Measure o)
	{
		return o.getName().compareTo(this.getName());
	}

	@Override
	public double getCombinedMeasureValue(double leftValue, double rightValue)
	{
		return Math.min(leftValue, rightValue);
	}

}
