package edu.emory.mathcs.measures;

import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;

public class NormalizedMeasure implements Measure {

	private String name; 
	private Measure m;
	private double min;
	private double max;
	
	public NormalizedMeasure(String name, Measure m, double min, double max)
	{
		this.name = name;
		this.m = m;
		this.min = min;
		this.max = max;
	}
	
	public double getMaxValue(final Tuple[] tuples) {
		return this.max;
	}

	public double getMinValue(final Tuple[] tuples) {
		return this.min;
	}

	@Override
	public String getName() {
		return this.name;
	}

//	@Override
//	public double getPossibleMaxValue() {
//		return -1.0;
//	}
//
//	@Override
//	public double getPossibleMaxValue(int numTuples) {
//		return -1.0;
//	}
//
//	@Override
//	public double getPossibleMinValue() {
//		return -1.0;
//	}
//
//	@Override
//	public double getPossibleMinValue(int numTuples) {
//		return -1.0;
//	}

	@Override
	public double getValue(TuplesGroup tuples) {
		
		double minValue = m.getMinValue(tuples.getTuples());
		double maxValue = m.getMaxValue(tuples.getTuples());
		
		double normalized01 = (m.getValue(tuples) - minValue) / (maxValue - minValue);
		
		return normalized01 * (this.max - this.min) + this.min;
	}

	@Override
	public int compareTo(Measure o) {
		return o.getName().compareTo(this.getName());
	}

	@Override
	public double getBetterScore(double score1, double score2) {
		return this.m.getBetterScore(score1, score2);
	}

	@Override
	public double getCombinedMeasureValue(double leftValue, double rightValue) {
		return Double.NaN;
	}

}
