package edu.emory.mathcs.measures.privacy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;

public class DisclosureProbability
{
	
	@Deprecated
	public static Map<Tuple, Double> getDisclosureProbabilities_old(TuplesGroup tuples) {
		Map<Tuple, Double> probs = new HashMap<Tuple, Double>();
		List<Tuple> ts = tuples.getTuplesAsList();
		
		if (ts == null)
			return probs;
		
		for (Tuple t : ts){
			if (t.isCorrupted())
				continue;
			
			probs.put(t, DisclosureProbability.getDisclosureProbability(tuples, t));
		}
		
		return probs;
	}

	public static Map<Tuple, Double> getDisclosureProbabilities(TuplesGroup tuples) {
		Map<Tuple, Double> probs = new HashMap<Tuple, Double>();
		
		if (tuples.hasSubGroups()) {
			probs.putAll(getDisclosureProbabilities(tuples.getLeftChild()));
			probs.putAll(getDisclosureProbabilities(tuples.getRightChild()));
		}
		else {
			Map<Integer, Integer> value2freq = tuples.getAttributeFrequenciesUncorrupted(TuplesGroup.SENSITIVE);
			int numTuples = 0;
			for (Integer v: value2freq.values())
				numTuples += v;

			if (numTuples == 0)
				return probs;

			for (Tuple t : tuples.getTuples()){
				if (t.isCorrupted()) {
					continue;
				}
				
				Integer freq = value2freq.get(t.getSensitiveData());
				if (freq == null)
					freq = 0;
				double probability = (double)freq / (double) numTuples;
				
				probs.put(t, probability);
			}
		}
		return probs;
	}

	private static double getDisclosureProbability(TuplesGroup tuples, Tuple tuple) {
		double probability = 0.0;
		
		List<Tuple> equivalenceGroup = getEquivalentUncorruptedTuples(tuples, tuple);
		
		if (equivalenceGroup.size() == 0)
			return Double.NaN;
		
		int freq = 0;
		for (Tuple t : equivalenceGroup)
		{
			if (tuple.getSensitiveData() == t.getSensitiveData())
				freq++;
		}
		probability = (double) freq / (double) equivalenceGroup.size();
		
		return probability;
	}

	private static List<Tuple> getEquivalentUncorruptedTuples(TuplesGroup tuples, final Tuple tuple)
	{
		List<Tuple> equivalenceTuples = new ArrayList<Tuple>();
		Stack<TuplesGroup> tg = new Stack<TuplesGroup>();
		tg.push(tuples);
		
		while (!tg.isEmpty())
		{
			tuples = tg.pop();
			if (tuples.hasSubGroups())
			{
				tg.push(tuples.getLeftChild());
				tg.push(tuples.getRightChild());
			}
			else
			{
				 if (tuples.doOverlap(tuple))
					 equivalenceTuples.addAll(tuples.getUncorruptedTuplesAsList());
			}
		}
		
		return equivalenceTuples;
	}
}
