package edu.emory.mathcs.measures.privacy;

import java.util.Map;

import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.measures.Measure;

public class DisclosureProbabilityMax implements Measure
{
	public static final String NAME = "DisclosureProbabilityMax";
	
	@Override
	public double getBetterScore(double score1, double score2)
	{
		return Math.min(score1, score2);
	}

	@Override
	public double getCombinedMeasureValue(double leftValue, double rightValue)
	{
		return Math.max(leftValue, rightValue);
	}

	@Override
	public double getMaxValue(Tuple[] tuples)
	{
		return Double.NaN;
	}

	@Override
	public double getMinValue(Tuple[] tuples)
	{
		return Double.NaN;
	}

	@Override
	public String getName()
	{
		return DisclosureProbabilityMax.NAME;
	}

	@Override
	public double getValue(TuplesGroup tuples)
	{
		Map<Tuple, Double> probs = DisclosureProbability.getDisclosureProbabilities(tuples);
		double max = Double.NaN;
		for (Double value : probs.values()) {
			if (Double.isNaN(max) || max < value)
				max = value;
		}
		
		return max;
	}

	@Override
	public int compareTo(Measure o)
	{
		return o.getName().compareTo(this.getName());	
	}
}
