package edu.emory.mathcs.measures.utility;

import java.util.Map;

import edu.emory.mathcs.Site;
import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.measures.Measure;

public class ModifiedEntropy implements Measure {

		public static final String NAME = "ModifiedEntropy";

		public double getValue(final TuplesGroup tuples) {

			double entropy = 0.0;

			if (tuples.hasSubGroups()) {
				entropy = this.getCombinedMeasureValue(this.getValue(tuples.getLeftChild()), this.getValue(tuples.getRightChild()));
			} else {
				Map<Integer, Integer> uncoruptedFrequencies = tuples.getAttributeFrequenciesUncorrupted(TuplesGroup.SENSITIVE);
				Map<Site, Integer> uncoruptedSitesFrequencies = tuples.getSiteFrequencies();
				
				if (uncoruptedSitesFrequencies.size() > 1)
				{
					double sumFreqs = 0;
					for (Integer i: uncoruptedSitesFrequencies.values())
						sumFreqs += i;
	
					if (sumFreqs == 0.0)
						return Double.NaN;
	
					// for each tuple
					for (Integer freq : uncoruptedFrequencies.values())
						entropy += getEntropyComponent(freq, sumFreqs);
				}
				else
					entropy = uncoruptedFrequencies.size() * getMaxEntropyComponent();
			}

			return entropy;
		}

		protected double getEntropyComponent(double value, double sum) {
			if (value == 0.0)
				return Double.NaN;

			return -value / sum * Math.log(value / sum);
		}
		
		protected double getMaxEntropyComponent()
		{
			return 1.0 / Math.E;
		}

		@Override
		public String getName() {
			return Entropy.NAME;
		}

		@Override
		public int compareTo(Measure o) {
			return o.getName().compareTo(this.getName());
		}

		@Override
		public double getBetterScore(double score1, double score2) {
			return Math.max(score1, score2);
		}

		// public double getPossibleMaxValue() {
		// return this.getPossibleMaxValue(2);
		// }
		//
		// public double getPossibleMinValue() {
		// return 0.0;
		// }
		//
		// public double getPossibleMaxValue(int numTuples) {
		// return this.getEntropyComponent(1.0, (double) numTuples) * (double)
		// numTuples;
		// }
		//
		// public double getPossibleMinValue(int numTuples) {
		// if (numTuples <= 0)
		// return Double.NaN;
		//		
		// return 0.0;
		// }

		/**
		 * Max value - all tuples are in one group
		 */
		public double getMaxValue(final Tuple[] tuples) {
			if (tuples.length == 0)
				return Double.NaN;

			return this.getEntropyComponent(1.0, (double) tuples.length) * (double) tuples.length;
		}

		/**
		 * Min value - each tuples is in separate group
		 */
		public double getMinValue(final Tuple[] tuples) {
			
			if (tuples.length == 0)
				return Double.NaN;
			
			return 0.0;
		}

		@Override
		public double getCombinedMeasureValue(double leftValue, double rightValue) {
			return leftValue + rightValue;
		}
	}

