package edu.emory.mathcs.measures.utility;

import java.util.List;

import edu.emory.mathcs.IntAttributeRange;
import edu.emory.mathcs.Query;
import edu.emory.mathcs.QueryRange;
import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.measures.Measure;

public class QueryError implements Measure {

	public static final String NAME = "QueryError";
	public static enum generation {RANDOM};
	
	private List<Query> queries;

	public QueryError(List<Query> queries) {
		this.queries = queries;
	}

	@Override
	public double getBetterScore(double score1, double score2) {
		return Math.min(score1, score2);
	}

	@Override
	public double getCombinedMeasureValue(double leftValue, double rightValue) {
		return Double.NaN;
	}

	@Override
	public double getMaxValue(Tuple[] tuples) {
		return (double) tuples.length;
	}

	@Override
	public double getMinValue(Tuple[] tuples) {
		return 0.0;
	}

	@Override
	public String getName() {
		return QueryError.NAME;
	}

	@Override
	public int compareTo(Measure arg0) {
		return arg0.getName().compareTo(this.getName());
	}

	@Override
	public double getValue(TuplesGroup tuples) {
		
		double qe = 0.0;
		int counter = 0;
		int numTuples = tuples.getNumTuples();
		
		for (Query query : this.queries)
		{
			int exact = this.getExact(tuples, query);
			int anonymized = this.getAnonymized(tuples, query);

			if (exact == 0)
				continue;
			
			//qe += (Math.abs(exact - anonymized) / (double)exact);
			qe += (Math.abs(exact - anonymized) / (double)numTuples);
			counter++;
		}
		
		return qe / (double) counter;
	}

	private int getAnonymized(TuplesGroup group, Query query) {
		if (group.hasSubGroups() == false) {
			int[] intersection = new int[query.getNumRanges()];
			for (int i = 0; i < intersection.length; i++) {
				intersection[i] = getRangeIntersect(query.getRange(i), group.getAttributeRange(query.getRange(i).getAttribute()));
			}
			double volTotal = 1;
			for (int i = 0; i < query.getNumRanges(); i++) {
				volTotal *= group.getAttributeRange(query.getRange(i).getAttribute()).getRange();
			}
			double volIntersect = 1;
			for (int i = 0; i < intersection.length; i++) {
				volIntersect *= intersection[i];
			}
			double fraction = volIntersect / (double)volTotal;
			return (int) Math.round(fraction * group.getNumTuples());
		} else {
			return this.getAnonymized(group.getLeftChild(), query) + this.getAnonymized(group.getRightChild(), query);
		}
	}

	private int getRangeIntersect(QueryRange queryRange, IntAttributeRange intAttributeRange) {
		int xA = queryRange.getMin();
		int yA = queryRange.getMax();
		int xB = intAttributeRange.getMin();
		int yB = intAttributeRange.getMax();
		if (xB > yA || xA > yB) {
			return 0;
		} else if (xB >= xA) {
			return Math.min(yA - xB, yB - xB);
		} else if (xA >= xB) {
			return Math.min(yB - xA, yA - xA);
		} else {
			throw new RuntimeException("Error");
		}
	}

	private int getExact(TuplesGroup group, Query query) {
		if (group.hasSubGroups() == false) {
			int cnt = 0;
			main: for (Tuple tuple : group.getTuples()) {
				if (tuple.isCorrupted())
					continue;
				for (int j = 0; j < query.getNumRanges(); j++) {
					if (!query.getRange(j).isInTheQueryRange(tuple)) {
						continue main;
					}
				}
				cnt++;
			}
			return cnt;
		} else {
			return this.getExact(group.getLeftChild(), query) + this.getExact(group.getRightChild(), query);
		}
	}

}
