package edu.emory.mathcs.privacy;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import edu.emory.mathcs.TuplesGroup;

public class AndCheckers implements PrivacyChecker {

	private Map<PrivacyChecker, Double> checkers;
	private String name;
	
	private AndCheckers(String name, Map<PrivacyChecker, Double> checkers)
	{
		this.name = name;
//		this.checkers = Collections.synchronizedMap(new LinkedHashMap<PrivacyChecker, Double>(checkers.size(), (float) 0.75, true));
		this.checkers = new HashMap<PrivacyChecker, Double>();
		this.checkers.putAll(checkers);
		
		reduceCheckers();
		
	}
	
	private void reduceCheckers() {
	}
	
	public static class AndCheckersBuilder 
	{
		private Map<PrivacyChecker, Double> checkers;
		private double sumWeights;

		public AndCheckersBuilder()
		{
			this.checkers = new HashMap<PrivacyChecker, Double>();
			this.sumWeights = 0.0;
		}
		
		public AndCheckersBuilder addChecker(PrivacyChecker checker, double weight)
		{
			this.checkers.put(checker, weight);
			this.sumWeights += weight;
			return this;
		}

		public AndCheckersBuilder removeChecker(PrivacyChecker checker)
		{
			Double weight = this.checkers.remove(checker);
			if (weight != null)
				this.sumWeights -= weight;
			return this;
		}

		public AndCheckers build(String name)
		{
			Map<PrivacyChecker, Double> weightedCheckers = new HashMap<PrivacyChecker, Double>();
			for (PrivacyChecker c: this.checkers.keySet()) {
				weightedCheckers.put(c, (this.checkers.get(c) / this.sumWeights));
			}
			return new AndCheckers(name, weightedCheckers);
		}
		
	}
	
	@Override
	public boolean isAnonymized(TuplesGroup tuples) {
		for (PrivacyChecker p: this.checkers.keySet()) {
			this.checkers.get(p); // to update linked list in the hash map - performance
			if (!p.isAnonymized(tuples))
				return false;
		}
		
		return true;
	}

	public String getName()
	{
		return this.name;
	}

	@Override
	public boolean isMonotonic()
	{
		for (PrivacyChecker pc : this.checkers.keySet())
			if (pc.isMonotonic() == false)
				return false;
		
		return true;
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		double privacyFitness = 0.0;
		final Set<PrivacyChecker> checkers = this.checkers.keySet();
		Map<PrivacyChecker, Double> checker2fintess = new HashMap<PrivacyChecker, Double>();
		
		boolean isPrivate = true; // if any fitness is < 1.0, then it's not private!
		double fitness = 0.0;
		for (PrivacyChecker p: checkers) {
			fitness = p.getPrivacyFitness(tuples);
			checker2fintess.put(p, fitness);
			isPrivate = isPrivate && (fitness >= 1.0); 
		}

		for (PrivacyChecker p: checkers) {
			fitness = checker2fintess.get(p);
			// for non-private case don't let to use overfitness in one attribute
			// to fill it in the other.
			if (!isPrivate)
				fitness = Math.min(0.99999, fitness);
			
			privacyFitness += (fitness * this.checkers.get(p));
		}
		
		return privacyFitness;
	}
	
}
