package edu.emory.mathcs.privacy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import edu.emory.mathcs.TuplesGroup;

public class CL_diversity implements PrivacyChecker {

	public static final String NAME = "CL_diversity";

	private int l;
	private double c;

	public CL_diversity(int l, double c) {
		this.l = l;
		this.c = c;
	}
	
	@Override
	public boolean isAnonymized(TuplesGroup tuples) {
		boolean isAnonymized = false;
		
		if (tuples.hasSubGroups() == true)
			isAnonymized = this.isAnonymized(tuples.getLeftChild()) && this.isAnonymized(tuples.getRightChild());
		else
			isAnonymized = this.isGroupcLdiverse(tuples);
		
		return isAnonymized;	
	}

	protected boolean isGroupcLdiverse(TuplesGroup tuples)
	{
		Map<Integer, Integer> freq = tuples.getAttributeFrequenciesUncorrupted(TuplesGroup.SENSITIVE);
	
		List<Integer> sortedFreq = new ArrayList<Integer>(freq.values());
		Collections.sort(sortedFreq);
		Collections.reverse(sortedFreq);
		
		int n1 = sortedFreq.get(0);
		int n_l = this.addElementsFrom(this.l - 1, sortedFreq);
		
		boolean iscLdiverse = (n1 <= this.c * n_l);
		
		return iscLdiverse;
	}

	protected int addElementsFrom(int from, List<Integer> sortedFreq) {
		int n_l = 0;
		for (int i = from; i < sortedFreq.size(); i++)
			n_l += sortedFreq.get(i);
		return n_l;
	}

	@Override
	public boolean isMonotonic()
	{
		return false;
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		double privacyLevel = 0.0;

		if (tuples.hasSubGroups() == false)
		{
			Map<Integer, Integer> freq = tuples.getAttributeFrequenciesUncorrupted(TuplesGroup.SENSITIVE);
			
			List<Integer> sortedFreq = new ArrayList<Integer>(freq.values());
			Collections.sort(sortedFreq);
			Collections.reverse(sortedFreq);
			
			int n1 = sortedFreq.get(0);
			int n_l = this.addElementsFrom(this.l - 1, sortedFreq);
			
			privacyLevel = this.c * n_l / n1;
		}
		else
		{
			privacyLevel = Math.min(this.getPrivacyFitness(tuples.getLeftChild()), this.getPrivacyFitness(tuples.getRightChild()));
		}

		return privacyLevel;
	}
}
