package edu.emory.mathcs.privacy;

import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.measures.Measure;
import edu.emory.mathcs.measures.MeasuresFactory;
import edu.emory.mathcs.measures.utility.Entropy;

public class Entropy_L_diversity implements PrivacyChecker {

	public static final String NAME = "Entropy_L_diversity";
	private double l;
	protected Measure entropyMeasure;
	
	public Entropy_L_diversity(double l)
	{
		this.l = l;
		this.entropyMeasure = new Entropy();
	}
	
	@Override
	public boolean isAnonymized(TuplesGroup tuples) {
		
		boolean isAnonymized = false;
		
		if (tuples.hasSubGroups() == true)
		{
			isAnonymized = this.isAnonymized(tuples.getLeftChild()) && this.isAnonymized(tuples.getRightChild());
		}
		else
		{
			double entropy = entropyMeasure.getValue(tuples);
			isAnonymized = (entropy >= Math.log(this.l));
		}
		
		return isAnonymized;
	}

	@Override
	public boolean isMonotonic()
	{
		return true;
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		double privacyFitness = 0.0;

		if (tuples.hasSubGroups() == false) {
			privacyFitness = Math.exp(this.entropyMeasure.getValue(tuples)) / (double) this.l;
		} else {
			privacyFitness = Math.min(this.getPrivacyFitness(tuples.getLeftChild()),
									  this.getPrivacyFitness(tuples.getRightChild()));
		}

		return privacyFitness;
	}

}
