package edu.emory.mathcs.privacy;

import edu.emory.mathcs.TuplesGroup;

public class K_anonymity implements PrivacyChecker {

	public static final String NAME = "K_anonymity";
	private int k;
	
	public K_anonymity(int k)
	{
		this.k = k;
	}
	
	public boolean isAnonymized(TuplesGroup tuples) {
		
		boolean isAnonymized = false;
		
		if (tuples.hasSubGroups() == false)
		{
			isAnonymized = (tuples.getNumUncorruptedTuples() >= this.k);
		}
		else
		{
			isAnonymized = this.isAnonymized(tuples.getLeftChild()) && this.isAnonymized(tuples.getRightChild());
		}
		
		return isAnonymized;
	}

	@Override
	public boolean isMonotonic()
	{
		return true;
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		double privacyFitness = 0.0;

		if (tuples.hasSubGroups() == false) {
			privacyFitness = (double) tuples.getNumUncorruptedTuples() / (double) this.k;
		} else {
			privacyFitness = Math.min(this.getPrivacyFitness(tuples.getLeftChild()),
									  this.getPrivacyFitness(tuples.getRightChild()));
		}

		return privacyFitness;
	}
}
