package edu.emory.mathcs.privacy;

import edu.emory.mathcs.TuplesGroup;

public class L_site_diversity implements PrivacyChecker {
	
	public static final String NAME = "L_site_diversity";
	private int l;
	
	public L_site_diversity(int l)
	{
		this.l = l;
	}
	
	@Override
	public boolean isAnonymized(TuplesGroup tuples) {
		
		boolean isAnonymized = false;
		
		if (tuples.hasSubGroups() == true)
		{
			isAnonymized = this.isAnonymized(tuples.getLeftChild()) && this.isAnonymized(tuples.getRightChild());
		}
		else
		{	// there is no subgroups - we need to check if there are l distinct sensitive attributes.
			int numSites = tuples.getSiteFrequencies().size();
			isAnonymized = (numSites >= this.l);
		}
		
		return isAnonymized;
	}

	@Override
	public boolean isMonotonic()
	{
		return true;
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		double privacyLevel = 0.0;
		
		if (tuples.hasSubGroups() == false)
		{
			int numSites = tuples.getSiteFrequencies().size();
			privacyLevel = numSites / (double) this.l;
		}
		else
		{
			privacyLevel = Math.min(this.getPrivacyFitness(tuples.getLeftChild()), this.getPrivacyFitness(tuples.getRightChild()));
		}
		
		return privacyLevel;
	}
}
