package edu.emory.mathcs.privacy;

import java.util.Arrays;

import edu.emory.mathcs.TuplesGroup;

public class M_privacy implements PrivacyChecker
{
	private M_privacy_BIN mPrivacyBIN;
	private double threshold;
//	private M_privacy_Sequence mPrivacyTopDown1;
	private M_privacy_Sequence_threads_pipelining mPrivacyTopDown;
	private PrivacyChecker checker;
	private int m;

	public M_privacy(final int m, PrivacyChecker privacyChecker, double threshold, int numThreads)
	{
		this.mPrivacyBIN = new M_privacy_BIN(privacyChecker, m, true, false);
		this.threshold = threshold;
		this.checker = privacyChecker;
		this.m = m;
//		this.mPrivacyTopDown1 = new M_privacy_Sequence(privacyChecker, m, Arrays.asList(m), true, false, false);
		this.mPrivacyTopDown = new M_privacy_Sequence_threads_pipelining(privacyChecker, m, Arrays.asList(m), true, numThreads); 
//		this.mPrivacyTopDown = new M_privacy_Sequence_threads(privacyChecker, m, Arrays.asList(m), true, numThreads);
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		return this.checker.getPrivacyFitness(tuples);
	}

	@Override
	public boolean isAnonymized(TuplesGroup tuples)
	{
		boolean isAnonymized = false;
		int numSites = tuples.getSites().size();
		
		if (numSites == 1)
			return this.checker.isAnonymized(tuples);
		
		if (this.checker.getPrivacyFitness(tuples) / (double)numSites >= this.threshold) {
			// top-down
			
			Integer[] sequenceOfAttackingCoalitions = new Integer[numSites];
			for (int i = numSites - 1; i >= 0; i--)
				sequenceOfAttackingCoalitions[numSites - i - 1] = this.m;
			
			this.mPrivacyTopDown.setSequenceOfAttackingCoalitions(Arrays.asList(sequenceOfAttackingCoalitions));
//			this.mPrivacyTopDown1.setSequenceOfAttackingCoalitions(Arrays.asList(sequenceOfAttackingCoalitions));
			isAnonymized = this.mPrivacyTopDown.isAnonymized(tuples);
//			boolean isAnonymized1 = this.mPrivacyTopDown1.isAnonymized(tuples);
//			
//			if (isAnonymized != isAnonymized1) {
//				System.out.print(isAnonymized + ":" + isAnonymized1 + ":" + tuples.getNumTuples() + ":" + tuples.getNumUncorruptedTuples());
//				tuples.printAllGroups(System.out);
//				isAnonymized = this.mPrivacyTopDown.isAnonymized(tuples);
//				isAnonymized1 = this.mPrivacyTopDown1.isAnonymized(tuples);
//			}
			
		}
		else {
		// BIN
			isAnonymized = this.mPrivacyBIN.isAnonymized(tuples);
//			System.out.println("bin");
		}
		
		return isAnonymized;

	}

	@Override
	public boolean isMonotonic()
	{
		// TODO Auto-generated method stub
		return false;
	}

	public PrivacyChecker getPrivacyChecker()
	{
		return this.checker;
	}

}
