package edu.emory.mathcs.privacy;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import edu.emory.mathcs.CombinationsGenerator;
import edu.emory.mathcs.CorruptionAttack;
import edu.emory.mathcs.Site;
import edu.emory.mathcs.Tuple;
import edu.emory.mathcs.TuplesGroup;

public class M_privacy_Sequence_threads implements PrivacyChecker
{
	public static long parallelTime = 0; // in ms
	private int m;
	private boolean withAdaptiveOrdering;
	private PrivacyChecker checker;
	private List<Integer> sequenceOfAttackingCoalitions;

	final private int numThreads;

	/**
	 * 
	 * @param m
	 * @param sequenceOfAttackingCoalitions sequence of attackers coalition sizes
	 * negative values counts from the end, i.e. -1 is a num of providers-1.
	 */
	@Deprecated
	public M_privacy_Sequence_threads(PrivacyChecker checker, int m,
			List<Integer> sequenceOfAttackingCoalitions, boolean withAdaptiveOrdering,
			int numThreads) {
		
		this.numThreads = numThreads;
		this.m = m;
		this.checker = checker;
		this.sequenceOfAttackingCoalitions = sequenceOfAttackingCoalitions;
		this.withAdaptiveOrdering = withAdaptiveOrdering;
	}
	
	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		return this.checker.getPrivacyFitness(tuples);
	}

	public static void sortSitesForAdaptiveOrdering(List<Site> sites, Map<Site, List<Tuple>>site2tuples, PrivacyChecker checker){
		Map<Site, Double> site2privacyLevel = new HashMap<Site, Double>();
		for (Site s: sites){
			TuplesGroup group = new TuplesGroup(site2tuples.get(s));
			site2privacyLevel.put(s, checker.getPrivacyFitness(group));
		}

		final Map<Site, Double> finalSite2privacyLevel = new HashMap<Site, Double>(site2privacyLevel);
		Collections.sort(sites, new Comparator<Site>() {
			@Override
			public int compare(Site site0, Site site1)
			{
				return -(int)Math.signum(finalSite2privacyLevel.get(site1) - finalSite2privacyLevel.get(site0));				
			}});
	}
	
	@Override
	public boolean isAnonymized(TuplesGroup tuples)
	{
		M_privacy_Sequence_threads.parallelTime -= Calendar.getInstance().getTimeInMillis();
		boolean isAnonymized = false;
		if (tuples.hasSubGroups()){
		
				isAnonymized = this.isAnonymized(tuples.getLeftChild()) && this.isAnonymized(tuples.getRightChild());
		 
		}
		else{
			Map<Site, List<Tuple>> site2tuples = tuples.getTuplesSplittedToSites();
			if (site2tuples.size() == 1)
				return this.checker.isAnonymized(tuples);
			try
			{
				isAnonymized = this.isAnonymized(tuples, site2tuples);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			catch (ExecutionException e)
			{
				e.printStackTrace();
			}		
		}
		
		M_privacy_Sequence_threads.parallelTime += Calendar.getInstance().getTimeInMillis();
		return isAnonymized;
	}

	
	private boolean isAnonymized(TuplesGroup tuples, Map<Site, List<Tuple>> site2tuples) throws InterruptedException, ExecutionException {
		List<Site> sites = new ArrayList<Site>(site2tuples.keySet());
		final int m = Math.min(this.m, sites.size() - 1);
		ExecutorService executor = Executors.newFixedThreadPool(this.numThreads);
		
		CombinationsGenerator<Site> revGenerator = null;
		if (this.withAdaptiveOrdering){
			sortSitesForAdaptiveOrdering(sites, site2tuples, this.checker);
			List<Site> revSites = new ArrayList<Site>(sites);
			Collections.reverse(revSites);
			revGenerator = new CombinationsGenerator<Site>(revSites);
		}
			
		CombinationsGenerator<Site> coalitionsGenerator = new CombinationsGenerator<Site>(sites);
		CombinationsGenerator<Site> mCoalitionsChecker = new CombinationsGenerator<Site>(sites);
		if (revGenerator == null)
			revGenerator = coalitionsGenerator;
		mCoalitionsChecker.setCardinality(m);
		
		for (int cardinality : this.sequenceOfAttackingCoalitions) {
			
			// interpret negative cardinality
			if (cardinality < 0)
				cardinality += sites.size();
			
			// for m > num_sites take m = num_sites - 1 (all but one providers collude)
			cardinality = Math.min(site2tuples.size() - 1, cardinality);
			if (cardinality < 0)
				continue;
			
			if (cardinality == 0) {
//				int numTuples = tuples.getNumTuples();
				IsPrivate isPrivateThread = new IsPrivate(tuples, new TreeSet<Site>(), this.checker);
				try
				{
					if (!isPrivateThread.call()) {
					//if (!this.isPrivate(tuples, new TreeSet<Site>())){
						// no need to do anything else :)
						executor.shutdown();
						return false;
					}
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
				continue;
			}

			coalitionsGenerator.setCardinality(cardinality);
			revGenerator.setCardinality(cardinality);
			while ((coalitionsGenerator.hasNext() || revGenerator.hasNext()) 
					&& !mCoalitionsChecker.areThisAndAllNextDecided()) {

				// for cardinality >= m use weaker attackers first!
//				Set<Site> attackingSites = (cardinality >= m ? coalitionsGenerator.next() : revGenerator.next());
//				if (attackingSites == null)
//					throw new IllegalArgumentException("Generators are not synchronized.");
				
				// privacy check
				// TODO: consider non-monotonic privacy constraints
				// use 'corrupted' and introduce 'excluded' concepts to
				// TuplesGroup, which needs to be reviewed.
				//Map<Set<Site>, Boolean> attackingSites2isPrivate = Collections.synchronizedMap(new HashMap<Set<Site>, Boolean>());
				Set<Set<Site>> attackingCombinations = (cardinality >= m ? coalitionsGenerator.nextCombinations(this.numThreads) : revGenerator.nextCombinations(this.numThreads));
				List<Set<Site>> attackingCombination = new ArrayList<Set<Site>>(attackingCombinations);
				List<Callable<Boolean>> threads = new ArrayList<Callable<Boolean>>(this.numThreads);
				
				for(Set<Site> attackingSites : attackingCombination) {
					threads.add(new IsPrivate(tuples, attackingSites, this.checker));
				//	attackingSites2isPrivate.put(attackingSites, this.executor.submit(isPrivateThread).get());
				}
				List<Future<Boolean>> results = executor.invokeAll(threads);

				for (int i = 0; i < threads.size(); i++) {
					Set<Site> attackingSites = attackingCombination.get(i);
					if (results.get(i).get() == false) {
	//				if (!this.isPrivate(tuples, attackingSites)){
						coalitionsGenerator.forbidThisAndAllSuper(attackingSites);
						revGenerator.forbidThisAndAllSuper(attackingSites);
						if (cardinality <= m) {
							executor.shutdown();
							return false; // early stop
						}
						else
							// just to keep generators synchronized, may be skipped
							mCoalitionsChecker.forbidThisAndAllSuper(attackingSites);
					}
					else {
						coalitionsGenerator.forbidThisAndAllSub(attackingSites);
						revGenerator.forbidThisAndAllSub(attackingSites);
						mCoalitionsChecker.forbidThisAndAllSub(attackingSites);
					}
				}
			}
			
			if (mCoalitionsChecker.areThisAndAllNextDecided())
				break;
		}
		
		executor.shutdown();
		return true;
	}
	
	public void setSequenceOfAttackingCoalitions(final List<Integer> newSequence){
		this.sequenceOfAttackingCoalitions = newSequence;
	}

	@Override
	public boolean isMonotonic()
	{
		// it's not entirely true, but for now it's enough
		return this.checker.isMonotonic();
	}
	
	private class IsPrivate implements Callable<Boolean> {
		private TuplesGroup tuples;
		private Set<Site> attackers;
		private PrivacyChecker checker;
		
		public IsPrivate(TuplesGroup tuples, Set<Site> attackers, PrivacyChecker checker) {
			this.tuples = tuples;
			this.attackers = attackers;
			this.checker = checker;
		}
		
		@Override
		public Boolean call() throws Exception
		{
			final int numSites = tuples.getSites().size();
			
			// number of attackers == number of sites in the tuples group
			if (numSites == attackers.size())
				return true;
			
			if (numSites <= 1)
				return this.checker.isAnonymized(tuples);
			
			int numCorruptedTuples = 0;
			int numUncorruptedTuples = 0;
			boolean isAnonymized = false;
			//System.out.println(this.)
			{
				Map<Site, List<Tuple>> site2tuples = this.tuples.getTuplesSplittedToSites();
				Set<Tuple> uncorruptedTuples = new HashSet<Tuple>();
				for (Site s: site2tuples.keySet()) {
					if (!attackers.contains(s))
						uncorruptedTuples.addAll(site2tuples.get(s));
				}
				TuplesGroup group = new TuplesGroup(uncorruptedTuples);
				
//				numCorruptedTuples = this.tuples.getNumTuples() - uncorruptedTuples.size();
//				numCorruptedTuples = CorruptionAttack.corruptSitesTuples(tuples, attackers);
		
				isAnonymized = this.checker.isAnonymized(group);
				
//				numUncorruptedTuples = CorruptionAttack.uncorruptSitesTuples(tuples, attackers);
//				numUncorruptedTuples = uncorruptedTuples.size();
			}
			
//			if (numCorruptedTuples != numUncorruptedTuples)
//				throw new IllegalArgumentException("Number of corrupted tuples: " + numCorruptedTuples
//						+ ", while number of uncorrupted tuples: " + numUncorruptedTuples);
			
			return isAnonymized;
		}

	}
}
