package edu.emory.mathcs.privacy;

import java.util.List;
import java.util.Map;

import edu.emory.mathcs.TuplesGroup;
import edu.emory.mathcs.Site;
import edu.emory.mathcs.Tuple;

public class Site_PrivacyChecker implements PrivacyChecker {

	private PrivacyChecker siteChecker;
	
	public Site_PrivacyChecker (PrivacyChecker siteChecker)
	{
		this.siteChecker = siteChecker;
	}
	
	@Override
	public boolean isAnonymized(TuplesGroup tuples) {
		
		Map<Site, List<Tuple>> site2Tuples = tuples.getTuplesSplittedToSites();

		for (List<Tuple> siteTuples : site2Tuples.values())
		{
			TuplesGroup group = new TuplesGroup(siteTuples.toArray(new Tuple[0]));
			if (this.siteChecker.isAnonymized(group) == false)
				return false;
		}
		
		return true;
	}

	@Override
	public boolean isMonotonic()
	{
		return this.siteChecker.isMonotonic();
	}

	@Override
	public double getPrivacyFitness(TuplesGroup tuples)
	{
		double privacyLevel = Double.MAX_VALUE;
		Map<Site, List<Tuple>> site2Tuples = tuples.getTuplesSplittedToSites();

		for (List<Tuple> siteTuples : site2Tuples.values())
		{
			TuplesGroup group = new TuplesGroup(siteTuples.toArray(new Tuple[0]));
			privacyLevel = Math.min(privacyLevel, this.siteChecker.getPrivacyFitness(group));
		}
		
		if (Double.MAX_VALUE == privacyLevel)
			return 0.0;
		return privacyLevel;
	}

}
